bl_info = {
    "name": "QUICK_UV_PRO",
    "author": "Yann Terrer",
    "version": (0, 3, 1),
    "blender": (2, 80, 0),
    "location": "3D Viewport",
    "description": "Handle Uvs from 3D viewport, 'B' keymap ",
    "warning": "",
    "doc_url": "",
    "category": "PIE MENU",
}

import bpy, math
import os
import bmesh
import addon_utils
import mathutils
from mathutils import Vector
from math import radians
import bpy.utils.previews
from bpy import context
from bpy.props import IntProperty, FloatProperty, FloatVectorProperty, StringProperty, BoolProperty, EnumProperty
import blf
from bpy_extras.io_utils import ImportHelper
import rna_keymap_ui
from . keymaps import get_hotkey_entry_item
from bpy.app.translations import pgettext_iface as iface_
from random import randint
from bpy_extras.node_utils import find_node_input

from . import auto_reaload
from . import custom_icons
from . import quick_image
from . import preferences
from . import keymaps
from . import qup_pie_menus
from . import light_texture
from . import light_preview
# from . import prop_gizmo
from . import terrain

from .UI.draw_text_modal import (draw_callback_px,  draw_modal_gizmo_world_text, draw_modal_gizmo_uv_text,draw_modal_displace)
from .custom_icons import get_icon_id


auto_reaload.init()



class QUP_OT_modal_uv(bpy.types.Operator):
    """Enter in Modal Uv to scale rotate and move Texture
Shortcut: Ctrl-MiddleMouse"""
    bl_idname = "qup.modal_uv"
    bl_label = "Quick Uv Pro: Modal and Gizmos"

    first_mouse_x: IntProperty()
    first_value: FloatProperty()

    user_active_color: FloatVectorProperty()
    user_editmesh_active: FloatVectorProperty(size=4)
    user_face_select: FloatVectorProperty(size=4)
    user_edge_select: FloatVectorProperty()
    user_wire_edit: FloatVectorProperty()
    @classmethod
    def poll(cls, context):
        return context.active_object is not None


    def modal(self, context, event):
        obj = context.object
        pass_through_events = {
            'WHEELUPMOUSE', 'WHEELDOWNMOUSE', 'LEFTMOUSE', 'BACK_SPACE', 'TAB',
            'NUMPAD_4', 'NUMPAD_6','NUMPAD_2','NUMPAD_8',
            'Q', 'I', 'U', 'F', 'E', 'A', 'K', 'W', 'P','Z','ONE','TWO','THREE','O',
        }
        if event.type in pass_through_events:
            return {'PASS_THROUGH'}
        if event.ctrl and event.type == 'Z' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.ctrl and event.type == 'R' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.shift and event.type == 'D' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.alt and event.type == 'LEFTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.ctrl and event.type == 'BACK_SPACE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.alt and event.type == 'RIGHTMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.shift and event.type == 'MIDDLEMOUSE' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.shift and event.type == 'MIDDLEMOUSE' and event.type == "SHIFT" and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.type == 'TAB' and event.value == 'PRESS':
            return {'PASS_THROUGH'}
        if event.type == 'NUMPAD_0' and event.value == 'PRESS':
            bpy.ops.uv.cube_project(cube_size=1, correct_aspect=False, clip_to_bounds=True, scale_to_bounds=False)

        if bpy.context.object.mode != "EDIT":
            try:
                bpy.ops.object.mode_set(mode="EDIT")

            except TypeError:
                pass

        else:
        
            try:
                try:
                    override_context = bpy.context.copy()
                    area = [area for area in bpy.context.screen.areas if area.type == "IMAGE_EDITOR"][-1]
                    override_context['window'] = bpy.context.window
                    override_context['screen'] = bpy.context.screen
                    override_context['area'] = area
                    override_context['region'] = area.regions[-1]
                    override_context['scene'] = bpy.context.scene
                    override_context['space_data'] = area.spaces.active

                    wm = bpy.context.window_manager
                    kc = wm.keyconfigs.user
                    km = kc.keymaps['3D View']
                    kmi = km.keymap_items['qup.modal_uv']
                    kmi2 = km.keymap_items['Quick Uv Pro: Validate Modal']

                    if event.type == 'T' and event.value == 'PRESS':
                        bpy.ops.qup.shortcut_popup('INVOKE_DEFAULT')

                    if event.type == 'X' and event.value == 'PRESS':
                        with context.temp_override(**override_context):
                            bpy.ops.transform.mirror( orient_type='GLOBAL',
                                                    orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)),
                                                    orient_matrix_type='GLOBAL', constraint_axis=(True, False, False))
                    if event.type == 'Y' and event.value == 'PRESS':
                        with context.temp_override(**override_context):
                            bpy.ops.transform.mirror( orient_type='GLOBAL',
                                                    orient_matrix=((1, 0, 0), (0, 1, 0), (0, 0, 1)),
                                                    orient_matrix_type='GLOBAL', constraint_axis=(False, True, False))
                    if event.type == 'S' and event.value == 'PRESS':
                        with context.temp_override(**override_context):
                            bpy.ops.transform.resize('INVOKE_DEFAULT')

                    if event.type == 'G' and event.value == 'PRESS':
                        with context.temp_override(**override_context):
                            bpy.ops.transform.translate('INVOKE_DEFAULT')
                    if event.type == 'R' and event.value == 'PRESS':
                        with context.temp_override(**override_context):
                            bpy.ops.transform.rotate( 'INVOKE_DEFAULT')

                    if event.type == 'B' and event.value == 'PRESS':
                        bpy.ops.wm.call_menu_pie(name="QUP_MT_pie_uv_modal")

                    if event.type == 'C' and event.value == 'PRESS':
                        old_area = bpy.context.area
                        old_type = old_area.type
                        for i, a in enumerate(bpy.context.screen.areas):
                            if a.type == 'IMAGE_EDITOR':
                                i = i
                                a = bpy.context.screen.areas[i]
                                s = a.spaces.active
                                s.uv_editor.lock_bounds = not s.uv_editor.lock_bounds
                                old_area.type = old_type
                        return {'RUNNING_MODAL'}

                    if event.type == 'MIDDLEMOUSE' and event.value == 'PRESS':
                        area3d = [area for area in bpy.context.screen.areas if area.type == "VIEW_3D"]
                        override_context['area'] = area3d
                        bpy.ops.view3d.rotate('INVOKE_DEFAULT')

                    override_context['area'] = area
                    
                    if event.type in {'RET', 'NUMPAD_ENTER', } or event.type == kmi2.type and event.value == 'PRESS':
                        context.preferences.themes[0].view_3d.editmesh_active = self.user_editmesh_active
                        context.preferences.themes[0].view_3d.face_select = self.user_face_select
                        context.preferences.themes[0].view_3d.edge_select = self.user_edge_select
                        context.preferences.themes[0].view_3d.wire_edit = self.user_wire_edit

                        try:
                            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                        except ValueError:
                            pass
                        with context.temp_override(**override_context):
                            bpy.ops.screen.area_close()


                        return {'FINISHED'}

                    elif event.ctrl == kmi.ctrl and event.type == kmi.type and event.value == 'PRESS':

                        context.preferences.themes[0].view_3d.editmesh_active = self.user_editmesh_active
                        context.preferences.themes[0].view_3d.face_select = self.user_face_select
                        context.preferences.themes[0].view_3d.edge_select = self.user_edge_select
                        context.preferences.themes[0].view_3d.wire_edit = self.user_wire_edit
                        try:
                            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                        except ValueError:
                            pass
                        with context.temp_override(**override_context):
                            bpy.ops.screen.area_close()
                except IndexError:
                    pass
                    return {'FINISHED'}

                if event.type in {'ESC'}:
                    context.preferences.themes[0].view_3d.editmesh_active = self.user_editmesh_active
                    context.preferences.themes[0].view_3d.face_select = self.user_face_select
                    context.preferences.themes[0].view_3d.edge_select = self.user_edge_select
                    context.preferences.themes[0].view_3d.wire_edit = self.user_wire_edit

                    try:
                        bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                    except ValueError:
                        pass
                    with context.temp_override(**override_context):
                        bpy.ops.screen.area_close()


                    return {'CANCELLED'}

            except Exception as e:
                context.preferences.themes[0].view_3d.editmesh_active = self.user_editmesh_active
                context.preferences.themes[0].view_3d.face_select = self.user_face_select
                context.preferences.themes[0].view_3d.edge_select = self.user_edge_select
                context.preferences.themes[0].view_3d.wire_edit = self.user_wire_edit
                raise e

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):

        self.user_editmesh_active = context.preferences.themes[0].view_3d.editmesh_active
        self.user_face_select  = context.preferences.themes[0].view_3d.face_select
        self.user_edge_select  = context.preferences.themes[0].view_3d.edge_select
        self.user_wire_edit    = context.preferences.themes[0].view_3d.wire_edit

        obj = context.object
        if obj.type == 'MESH':
            if bpy.context.object.mode == "EDIT":
                bpy.ops.object.editmode_toggle()

            try:
                tc = bpy.context.object.active_material.node_tree.nodes.get("Texture Coordinate", None)
                # if tc is not None and tc.object is None and len(tc.outputs[0].links)<0:
                #     pass
                # if tc is not None and len(tc.outputs[0].links)>0:
                #     bpy.ops.qup.handle_generated('INVOKE_DEFAULT')
                #     return {'FINISHED'}
                if tc is not None and tc.object is not None and tc.object.name.startswith('Gizmo_WORLD'):
                    bpy.ops.qup.select_world_gizmo('INVOKE_DEFAULT')
                    return {'FINISHED'}

                else:
                    for modifier in obj.modifiers:
                        if modifier.name.startswith("QuickUVProject_Cube"):
                            bpy.ops.qup.select_cube_gizmo('INVOKE_DEFAULT')
                            return {'FINISHED'}
                        if modifier.name.startswith("QuickUVProject_Plane"):
                            bpy.ops.qup.select_plane_gizmo('INVOKE_DEFAULT')
                            return {'FINISHED'}

            except AttributeError:
                self.report({'ERROR'}, "No Material Yet")
                return {'FINISHED'}
            except KeyError:
                pass

            if context.area.type == 'VIEW_3D' and context.object:

                args = (self, context)
                self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_callback_px, args, 'WINDOW', 'POST_PIXEL')

                op = bpy.context.preferences.addons[__package__].preferences.Face_opacity_Modal_Uv
                blue = bpy.context.preferences.addons[__package__].preferences.Text_Color_Modal_Uv

                bpy.context.preferences.themes[0].view_3d.editmesh_active = (blue[0], blue[1] - .3, blue[2], op - .01)
                bpy.context.preferences.themes[0].view_3d.face_select = (blue[0], blue[1], blue[2], op)
                bpy.context.preferences.themes[0].view_3d.edge_select = (blue[:-1])
                bpy.context.preferences.themes[0].view_3d.wire_edit = (.5, .5, .5)

                bpy.context.area.ui_type = 'UV'
                bpy.context.space_data.pivot_point = 'CENTER'

                bpy.ops.screen.area_split(direction='HORIZONTAL', factor=0.01, cursor=(0, 0))
                bpy.context.scene.tool_settings.use_uv_select_sync = True

                bpy.context.area.ui_type = 'VIEW_3D'

                context.window_manager.modal_handler_add(self)
                return {'RUNNING_MODAL'}

            else:
                self.report({'WARNING'}, "No active object, could not finish")
                return {'CANCELLED'}

        else:
            self.report({'ERROR'}, "Select a Mesh to Set Uv")
            return {'CANCELLED'}


class QUP_OT_uv_cube_proj(bpy.types.Operator):
    ''' Cube Projection '''
    bl_label = "UvCube"
    bl_idname = "qup.uv_cube"

    def execute(self, context):
        error_gizmo(self)

        if bpy.context.object.mode != "EDIT":

            bpy.ops.object.mode_set(mode="EDIT")
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.uv.cube_project(cube_size=.5,
                                    scale_to_bounds=True,
                                    correct_aspect=True)
            bpy.ops.object.editmode_toggle()

        else:
            bpy.ops.uv.cube_project(cube_size=2,
                                    scale_to_bounds=True,
                                    correct_aspect=True)

        return {'FINISHED'}


class QUP_OT_uv_from_view(bpy.types.Operator):
    '''Project From View'''
    bl_label = "Uv from View"
    bl_idname = "qup.uv_fromview"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event):
        error_gizmo(self)

        if bpy.context.object.mode == "EDIT":

            bpy.ops.uv.project_from_view(orthographic=False, camera_bounds=False, correct_aspect=True,
                                         scale_to_bounds=True)

        else:
            bpy.ops.object.mode_set(mode="EDIT")
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.uv.project_from_view(orthographic=False, camera_bounds=False, correct_aspect=True,
                                         scale_to_bounds=True)
            bpy.ops.object.editmode_toggle()

        return {'FINISHED'}


class QUP_OT_uv_cylinder_proj(bpy.types.Operator):
    '''Project only in Orthogonal View'''
    bl_label = "Uv cylinder"
    bl_idname = "qup.uv_cylinder"

    def execute(self, _context):
        error_gizmo(self)

        if bpy.context.object.mode != "EDIT":
            bpy.ops.object.mode_set(mode="EDIT")
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.uv.cylinder_project(direction='ALIGN_TO_OBJECT', clip_to_bounds=True, scale_to_bounds=True)

            bpy.ops.object.editmode_toggle()

        else:

            bpy.ops.uv.cylinder_project(direction='ALIGN_TO_OBJECT', clip_to_bounds=True, scale_to_bounds=True)

        return {'FINISHED'}


class QUP_OT_uv_sphere_proj(bpy.types.Operator):
    '''Sphere Projection'''
    bl_label = "Uv cylinder"
    bl_idname = "qup.uv_sphere"

    def execute(self, _context):
        error_gizmo(self)
        if bpy.context.object.mode != "EDIT":
            bpy.ops.object.mode_set(mode="EDIT")
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.qup.uv_sphere_project()
            bpy.ops.qup.uv_sphere_project(direction='ALIGN_TO_OBJECT', clip_to_bounds=True, scale_to_bounds=True)
            bpy.ops.object.editmode_toggle()

        else:
            bpy.ops.uv.sphere_project(direction='ALIGN_TO_OBJECT', clip_to_bounds=True, scale_to_bounds=True)


        return {'FINISHED'}


class QUP_OT_apply_mesh(bpy.types.Operator):
    '''Select detached Face, Select orgiginal Mesh and Click Apply'''
    bl_label = "Apply Mesh"
    bl_idname = "qup.apply_mesh"

    def execute(self, context):
        bpy.ops.object.convert(target='MESH')
        bpy.ops.object.join()
        bpy.ops.object.editmode_toggle()
        bpy.ops.mesh.select_mode(use_extend=False, use_expand=False, type='VERT')
        bpy.ops.mesh.select_all(action='SELECT')
        bpy.ops.mesh.remove_doubles()
        bpy.ops.object.editmode_toggle()
        bpy.ops.object.select_all(action='DESELECT')

        return {'FINISHED'}

class QUP_OT_apply_gizmo(bpy.types.Operator):
    '''Apply Gizmo'''
    bl_label = "Apply Gizmo"
    bl_idname = "qup.apply_gizmo"

    param_gizmo: StringProperty()

    def execute(self, context,):
        obj = bpy.context.object
        objects_in_collection = bpy.data.collections["UV_GIZMO"].all_objects

        if self.param_gizmo == 'C':
            for modifier in obj.modifiers:
                if modifier.name.startswith('QuickUVProject_Cube'):
                    bpy.ops.object.modifier_apply(modifier=modifier.name)
            Gizmos = [obj for obj in bpy.context.object.children_recursive
                      if obj in list(objects_in_collection) and obj.name.startswith("Gizmo_CUBE")]
            Gz = Gizmos[-1]
            for ob in Gz.children_recursive:
                bpy.data.objects.remove(ob)
            bpy.data.objects.remove(Gz)

        if self.param_gizmo == 'P':
            for modifier in obj.modifiers:
                if modifier.name.startswith('QuickUVProject_Plane'):
                    bpy.ops.object.modifier_apply(modifier=modifier.name)

            Gizmos = [obj for obj in bpy.context.object.children_recursive
                      if obj in list(objects_in_collection) and obj.name.startswith("Gizmo_PLANE")]
            Gz = Gizmos[-1]
            bpy.data.objects.remove(Gz)

        return {'FINISHED'}

def random_face(bm):
    bm.faces.ensure_lookup_table()
    selection = [f for f in bm.faces if f.select and len(f.verts) == 4]
    for f in selection:
        if f.select:
            return selection[randint(0, len(selection) - 1)]
class QUP_OT_uv_straight_unwrap(bpy.types.Operator):
    '''Automatic Unwrap and straight island\nBe Sure to Have Only Quads in your Mesh or Selection'''
    bl_label = "Straight Unwrap"
    bl_idname = "qup.uv_straight_unwrap"

    user_active_face: IntProperty
    user_selection: StringProperty
    user_active_object: StringProperty

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event):
        error_gizmo(self)
        if not addon_utils.enable("bl_ext.blender_org.magic_uv", default_set=True):
            addon_utils.enable("bl_ext.blender_org.magic_uv", default_set=True)

        obj = bpy.context.object

        if bpy.context.object.mode != "EDIT":

            bpy.ops.object.editmode_toggle()
            bm = bmesh.from_edit_mesh(obj.data)
            me = obj.data

            for f in bm.faces:
                if len(f.verts) == 4:
                    f.select = True
                    bm.faces.active = f
                    bm.faces.active.select = True
                    bmesh.update_edit_mesh(me)
                    bm.select_history.add(bm.faces.active)

            bpy.ops.mesh.select_mode(use_extend=False, use_expand=True, type='FACE')
            bpy.ops.mesh.select_all(action='SELECT')

            non_quads = [f for f in bm.faces if f.select and len(f.verts) != 4]
            if non_quads:
                bpy.ops.qup.warning_straight_unwrap('INVOKE_DEFAULT')
                for f in bm.faces:
                    if len(f.verts) == 4:
                        f.select = True
                    else:
                        f.select = False
                # return {'FINISHED'}

            bpy.ops.uv.lightmap_pack()
            try:
                bpy.ops.uv.follow_active_quads()
            except RuntimeError:
                self.report({'INFO'}, "Mesh has Tris, Only Quads are well Unwraped and Straightened")

            bpy.context.area.ui_type = 'UV'


            bpy.context.space_data.uv_editor.lock_bounds = True
            bpy.context.space_data.pivot_point = 'CENTER'
            bpy.ops.uv.pack_islands(rotate=True, margin=0.001)
            bpy.ops.transform.rotate(value=1.5708)
            bpy.ops.transform.resize(value=(300, 300, 300))
            bpy.context.space_data.pivot_point = 'CURSOR'
            bpy.ops.uv.muv_pack_uv(allowable_center_deviation=(6.21, 10), allowable_size_deviation=(10, 0.001),
                                   accurate_island_copy=True, stride=(5.25, 0), apply_pack_uv=True)
            bpy.ops.transform.resize(value=(300, 300, 300))
            bpy.context.space_data.uv_editor.lock_bounds = False


            bpy.context.area.ui_type = 'VIEW_3D'

            bpy.ops.object.editmode_toggle()

        if bpy.context.object.mode == "EDIT":

            bm = bmesh.from_edit_mesh(obj.data)
            me = obj.data
            
            bmesh.update_edit_mesh(me)

            # bpy.ops.object.face_map_add()
            # bpy.ops.object.face_map_assign()
            # bpy.ops.mesh.separate(type='SELECTED')
            # bpy.ops.object.editmode_toggle()
            # context.object.select_set(True)
            # bpy.ops.object.join()
            # bpy.ops.object.editmode_toggle()
            #
            bpy.ops.mesh.select_all(action='SELECT')
            bpy.ops.mesh.remove_doubles()
            bpy.ops.mesh.select_all(action='DESELECT')
            # bpy.ops.object.face_map_select()

            obj = context.object
            bm = bmesh.from_edit_mesh(obj.data)
            act_face = bm.faces.active
            if not act_face:
                bm.faces.active = random_face(bm)

            non_quads = [f for f in bm.faces if f.select and len(f.verts) != 4]

            act_face = bm.faces.active
            self.user_active_face = act_face
            if non_quads:

                bpy.ops.qup.warning_straight_unwrap('INVOKE_DEFAULT')

                if len(act_face.verts) != 4:
                    bm.faces.active = random_face(bm)
                    bm.faces.active.select = True

                # for f in bm.faces:
                for f in non_quads:
                    if len(f.verts) == 4:
                        f.select = True
                    else:
                        f.select = False

            # else:
            bpy.ops.uv.unwrap()
            bpy.ops.uv.lightmap_pack()

            try:
                bpy.ops.uv.follow_active_quads()
            except RuntimeError:
                self.report({'INFO'}, "Mesh has Tris, Only Quads were well Unwrapped and Straightened")
            #
            bpy.context.area.ui_type = 'UV'
            bpy.context.scene.tool_settings.use_uv_select_sync = False

            bpy.context.space_data.pivot_point = 'CENTER'
            bpy.context.space_data.uv_editor.lock_bounds = True
            bpy.ops.uv.pack_islands(rotate=True, margin=0.001)
            bpy.ops.transform.rotate(value=1.5708)
            bpy.ops.transform.resize(value=(300, 300, 300))
            bpy.context.space_data.pivot_point = 'CURSOR'
            bpy.ops.uv.muv_pack_uv(allowable_center_deviation=(6.21, 10), allowable_size_deviation=(10, 0.001),
                                   accurate_island_copy=True, stride=(5.25, 0), apply_pack_uv=True)
            bpy.ops.transform.resize(value=(300, 300, 300))
            bpy.context.space_data.uv_editor.lock_bounds = False

            bpy.context.area.ui_type = 'VIEW_3D'

            # bpy.ops.object.face_map_remove()

        return {'FINISHED'}



class QUP_OT_copy_material(bpy.types.Operator):
    '''Link Material'''
    bl_label = "CopyMaterial"
    bl_idname = "qup.copy_material"

    def execute(self, context):
        bpy.ops.object.make_links_data(type='MATERIAL')

        return {'FINISHED'}
class QUP_OT_rotate_uv_deg(bpy.types.Operator,):
    bl_label = "RotateUv"
    bl_idname = "qup.rotate_uv_deg"

    param_deg: StringProperty()
    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event,):
        error_gizmo(self)

        if bpy.context.object.mode != "EDIT":
            if bpy.context.scene.tool_settings.use_uv_select_sync != True:
                bpy.context.scene.tool_settings.use_uv_select_sync = True

            for d in (15, 30, 45, 90, 180):

                if self.param_deg == f'{d}':
                    bpy.ops.object.editmode_toggle()
                    bpy.context.area.ui_type = 'UV'
                    bpy.ops.uv.select_all(action='SELECT')
                    bpy.ops.transform.rotate(value=radians(d))
                    bpy.ops.object.editmode_toggle()
                    bpy.context.area.ui_type = 'VIEW_3D'
                if self.param_deg == f'-{d}':
                    bpy.ops.object.editmode_toggle()
                    bpy.context.area.ui_type = 'UV'
                    bpy.ops.uv.select_all(action='SELECT')
                    bpy.ops.transform.rotate( value=radians(-d))
                    bpy.ops.object.editmode_toggle()
                    bpy.context.area.ui_type = 'VIEW_3D'

        else:
            if bpy.context.scene.tool_settings.use_uv_select_sync != True:
                bpy.context.scene.tool_settings.use_uv_select_sync = True

            for d in (15, 30, 45, 90, 180):

                if self.param_deg == f'{d}':
                    bpy.context.area.ui_type = 'UV'
                    bpy.ops.transform.rotate(value=radians(d))
                    bpy.context.area.ui_type = 'VIEW_3D'
                if self.param_deg == f'-{d}':
                    bpy.context.area.ui_type = 'UV'
                    bpy.ops.transform.rotate(value=radians(-d))
                    bpy.context.area.ui_type = 'VIEW_3D'

        return {'FINISHED'}
class QUP_OT_rotate_uv(bpy.types.Operator):
    '''Rotation "90°"
Ctrl Rotation "45°"
Shift Rotation "30°"'''

    bl_label = "RotateUv"
    bl_idname = "uv.rotate"

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event):
        error_gizmo(self)

        if bpy.context.object.mode != "EDIT":
            if bpy.context.scene.tool_settings.use_uv_select_sync != True:
                bpy.context.scene.tool_settings.use_uv_select_sync = True

            if event.ctrl:
                bpy.ops.object.editmode_toggle()
                bpy.context.area.ui_type = 'UV'
                bpy.ops.uv.select_all(action='SELECT')
                bpy.ops.transform.rotate(value=radians(45))
                bpy.ops.object.editmode_toggle()
                bpy.context.area.ui_type = 'VIEW_3D'

            if event.shift:
                bpy.ops.object.editmode_toggle()
                bpy.context.area.ui_type = 'UV'
                bpy.ops.uv.select_all(action='SELECT')
                bpy.ops.transform.rotate(value=radians(30))
                bpy.ops.object.editmode_toggle()
                bpy.context.area.ui_type = 'VIEW_3D'


            else:
                bpy.ops.object.editmode_toggle()
                bpy.context.area.ui_type = 'UV'
                bpy.ops.uv.select_all(action='SELECT')
                bpy.ops.transform.rotate(value=radians(90))
                bpy.ops.object.editmode_toggle()
                bpy.context.area.ui_type = 'VIEW_3D'


        else:
            if bpy.context.scene.tool_settings.use_uv_select_sync != True:
                bpy.context.scene.tool_settings.use_uv_select_sync = True

            if event.ctrl:
                bpy.context.area.ui_type = 'UV'
                bpy.ops.transform.rotate(value=radians(45))
                bpy.context.area.ui_type = 'VIEW_3D'

            if event.shift:
                bpy.context.area.ui_type = 'UV'
                bpy.ops.transform.rotate(value=radians(30))
                bpy.context.area.ui_type = 'VIEW_3D'

            else:

                bpy.context.area.ui_type = 'UV'
                bpy.ops.transform.rotate(value=radians(90))
                bpy.context.area.ui_type = 'VIEW_3D'

        return {'FINISHED'}

class QUP_OT_align_view_to_face(bpy.types.Operator):
    '''Align view to selected face'''
    bl_label = "AlignViewToFace"
    bl_idname = "align.viewtoface"

    def execute(self, context):
        bpy.ops.view3d.view_axis(type='TOP', align_active=True)

        return {'FINISHED'}
class QUP_OT_modal_add_mat_nw(bpy.types.Operator, ImportHelper):
    """Create Multiple texture Shader"""
    bl_idname = "object.add_mat_nw"
    bl_label = "Modal add mat_nw"

    def modal(self, context, event):

        _timer = None

        override_context = bpy.context.copy()

        area = [area for area in bpy.context.screen.areas if area.type == "NODE_EDITOR"][0]
        override_context['window'] = bpy.context.window
        override_context['screen'] = bpy.context.screen
        override_context['area'] = area
        override_context['region'] = area.regions[-1]
        override_context['scene'] = bpy.context.scene
        override_context['space_data'] = area.spaces.active

        if event.type == 'TIMER':
            if bpy.context.object.mode != "EDIT":
                bpy.ops.object.editmode_toggle()
                bpy.ops.uv.cube_project(cube_size=.5, scale_to_bounds=True)
                bpy.ops.object.material_slot_assign()
                bpy.ops.object.editmode_toggle()
                with context.temp_override(**override_context):
                    bpy.ops.node.nw_add_textures_for_principled( 'INVOKE_DEFAULT')
                bpy.ops.object.material_slot_assign()

            else:
                bpy.ops.uv.cube_project(cube_size=.5, scale_to_bounds=True)
                bpy.ops.object.material_slot_assign()
                bpy.ops.object.editmode_toggle()

                with context.temp_override(**override_context):
                    bpy.ops.node.nw_add_textures_for_principled( 'INVOKE_DEFAULT')
                bpy.ops.object.editmode_toggle()
                bpy.ops.object.material_slot_assign()

            return {'FINISHED'}


        elif event.type in {'ESC'}:

            return {'CANCELLED'}

        return {'RUNNING_MODAL'}

    def invoke(self, context, event):
        head, tail = os.path.split(self.filepath)
        if context.object:

            area = [area for area in bpy.context.screen.areas if area.type == "NODE_EDITOR"]
            if len(area) == 0:

                bpy.context.area.ui_type = 'ShaderNodeTree'
                bpy.ops.screen.area_split(direction='HORIZONTAL', factor=0.01, cursor=(0, 0))

                bpy.context.area.ui_type = 'VIEW_3D'

                material_bascic = bpy.data.materials.new(name="Node_Wrangler Material")
                material_bascic.use_nodes = True

                bpy.context.object.active_material = material_bascic

                principled_node = material_bascic.node_tree.nodes.get('Principled BSDF')

                nodes = material_bascic.node_tree.nodes
                for node in nodes:
                    node.select = False

                principled_node.select = True
                nodes.active = principled_node

                wm = context.window_manager
                self._timer = wm.event_timer_add(0.1, window=context.window)
                wm.modal_handler_add(self)

                return {'RUNNING_MODAL'}


            else:
                if bpy.context.object.mode != "EDIT":

                    bpy.context.area.ui_type = 'VIEW_3D'

                    area = [area for area in bpy.context.screen.areas if area.type == "NODE_EDITOR"][0]

                    material_bascic = bpy.data.materials.new(name="Node_Wrangler Material")
                    material_bascic.use_nodes = True

                    bpy.context.object.active_material = material_bascic

                    principled_node = material_bascic.node_tree.nodes.get('Principled BSDF')

                    nodes = material_bascic.node_tree.nodes
                    for node in nodes:
                        node.select = False

                    principled_node.select = True
                    nodes.active = principled_node

                    wm = context.window_manager
                    self._timer = wm.event_timer_add(0.1, window=context.window)
                    wm.modal_handler_add(self)

                else:

                    bpy.ops.object.material_slot_add()
                    bpy.ops.object.material_slot_assign()
                    bpy.context.area.ui_type = 'VIEW_3D'
                    area = [area for area in bpy.context.screen.areas if area.type == "NODE_EDITOR"][0]

                    material_bascic = bpy.data.materials.new(name="Node_Wrangler Material")
                    material_bascic.use_nodes = True

                    bpy.context.object.active_material = material_bascic

                    principled_node = material_bascic.node_tree.nodes.get('Principled BSDF')

                    nodes = material_bascic.node_tree.nodes
                    for node in nodes:
                        node.select = False

                    principled_node.select = True
                    nodes.active = principled_node

                    wm = context.window_manager
                    self._timer = wm.event_timer_add(0.1, window=context.window)
                    wm.modal_handler_add(self)

                return {'RUNNING_MODAL'}

        else:
            self.report({'WARNING'}, "No active object, could not finish")
            return {'CANCELLED'}

class QUP_OT_add_one_texture_material(bpy.types.Operator, ImportHelper):
    """Create One texture Shader"""
    bl_idname = "qup.add_one_texture_material"
    bl_label = "add mat"


    def execute(self, context):
        filename, extension = os.path.splitext(self.filepath)
        head, tail = os.path.split(self.filepath)

        if bpy.context.object.mode != "EDIT":

            ######## APPEND One_Texture_Shader Node Group ############################
            file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")
            inner_path = 'NodeTree'
            nodetree_name = 'One_Texture_Shader'

            bpy.ops.wm.append(
                filepath=os.path.join(file_path, inner_path, nodetree_name),
                directory=os.path.join(file_path, inner_path),
                filename=nodetree_name
            )

            ######## Create New PRINCPLED BSDF ############################

            material_bascic = bpy.data.materials.new(name="New Mat")
            material_bascic.use_nodes = True

            bpy.context.object.active_material = material_bascic

            node_to_delete = material_bascic.node_tree.nodes['Principled BSDF']
            material_bascic.node_tree.nodes.remove(node_to_delete)

            Output_Node = material_bascic.node_tree.nodes.get('Material Output')

            One_Shader_Node = material_bascic.node_tree.nodes.new(type='ShaderNodeGroup')
            One_Shader_Node.node_tree = bpy.data.node_groups['One_Texture_Shader']
            One_Shader_Node.name = 'One_Texture_Shader'
            One_Shader_Node.label = 'One_Texture_Shader'

            One_Shader_Node.location = (-300, 215)

            Texture_Node = material_bascic.node_tree.nodes.new('ShaderNodeTexImage')

            Texture_Node.location = (-600, 215)

            material_bascic.node_tree.links.new(Texture_Node.outputs[0], One_Shader_Node.inputs[0])
            material_bascic.node_tree.links.new(Texture_Node.outputs[1], One_Shader_Node.inputs[8])
            material_bascic.node_tree.links.new(One_Shader_Node.outputs[0], Output_Node.inputs['Surface'])
            material_bascic.node_tree.links.new(One_Shader_Node.outputs[1], Output_Node.inputs['Displacement'])

            nodes = material_bascic.node_tree.nodes
            for node in nodes:
                node.select = False

            Texture_Node.select = True
            Output_Node.select = True
            nodes.active = Texture_Node

            img = bpy.data.images.load(self.filepath)
            Texture_Node.image = img

            my_object = bpy.context.view_layer.objects.active  # assign variable
            my_object.select_set(True)

            material_bascic.name = tail[:-4]

            return {'FINISHED'}

        else:

            bpy.ops.object.editmode_toggle()

            bpy.ops.object.material_slot_add()
            bpy.ops.object.material_slot_assign()

            file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")

            inner_path = 'NodeTree'
            nodetree_name = 'One_Texture_Shader'

            bpy.ops.wm.append(
                filepath=os.path.join(file_path, inner_path, nodetree_name),
                directory=os.path.join(file_path, inner_path),
                filename=nodetree_name
            )

            ######## Create New PRINCPLED BSDF ############################

            material_bascic = bpy.data.materials.new(name="New Mat")
            material_bascic.use_nodes = True

            bpy.context.object.active_material = material_bascic

            node_to_delete = material_bascic.node_tree.nodes['Principled BSDF']
            material_bascic.node_tree.nodes.remove(node_to_delete)

            Output_Node = material_bascic.node_tree.nodes.get('Material Output')

            One_Shader_Node = material_bascic.node_tree.nodes.new(type='ShaderNodeGroup')
            One_Shader_Node.node_tree = bpy.data.node_groups['One_Texture_Shader']

            One_Shader_Node.location = (-300, 215)

            Texture_Node = material_bascic.node_tree.nodes.new('ShaderNodeTexImage')

            Texture_Node.location = (-600, 215)

            material_bascic.node_tree.links.new(Texture_Node.outputs[0], One_Shader_Node.inputs[0])
            material_bascic.node_tree.links.new(Texture_Node.outputs[1], One_Shader_Node.inputs[8])
            material_bascic.node_tree.links.new(One_Shader_Node.outputs[0], Output_Node.inputs['Surface'])
            material_bascic.node_tree.links.new(One_Shader_Node.outputs[1], Output_Node.inputs['Displacement'])

            nodes = material_bascic.node_tree.nodes
            for node in nodes:
                node.select = False

            Texture_Node.select = True
            Output_Node.select = True
            nodes.active = Texture_Node

            img = bpy.data.images.load(self.filepath)
            Texture_Node.image = img
            bpy.ops.object.material_slot_assign()

            bpy.ops.object.editmode_toggle()
            bpy.ops.uv.cube_project(cube_size=1, correct_aspect=True)

            material_bascic.name = tail[:-4]

            bpy.ops.object.material_slot_assign()

            return {'FINISHED'}

class QUP_OT_handle_generated(bpy.types.Operator):
    """Handle Generated texture Shader"""
    bl_idname = "qup.handle_generated"
    bl_label = "Handle Generated texture"
    def execute(self, context):

        return {'FINISHED'}

    def invoke(self, context, event):

        return context.window_manager.invoke_popup(self, width=280,)

    def draw(self, context):

        Node_tree = bpy.context.object.active_material.node_tree
        Nodes = Node_tree.nodes
        tm_nodes = []
        for nodes in Nodes:
            if nodes.type == 'MAPPING':
                if len(nodes.outputs[0].links) > 0:
                    tm_nodes.append(nodes)

        layout = self.layout
        layout.label(text='Generated Texture Coordinates', icon='GRIP')

        layout.operator("qup.swap_textures", text='Swap Textures', icon='ARROW_LEFTRIGHT')


        for nodes in tm_nodes:

            if nodes.outputs[0].links[0].to_node.type == 'TEX_IMAGE':
                node_to = nodes.outputs[0].links[0].to_node
                texture = node_to.image


                tm = Nodes.get(nodes.name, None)

                tm_rot = tm.inputs[1]
                tm_loc = tm.inputs[2]
                tm_sca = tm.inputs[3]

                image = node_to.image
                image.preview_ensure()
                node_to.image.preview_ensure()

                box = layout.box()

                box.label(text=image.name, icon_value=image.preview.icon_id, )
                # box.column().label(text=texture.name)
                row = box.row()
                row.column().prop(tm_rot, "default_value", text="Location", )
                row.column().prop(tm_loc, "default_value", text="Rotation", )
                row.column().prop(tm_sca, "default_value", text="Scale", )

                # box = layout.box()

            else:
                texture = nodes.outputs[0].links[0].to_node
                tm = Nodes.get(nodes.name, None)

                tm_rot = tm.inputs[1]
                tm_loc = tm.inputs[2]
                tm_sca = tm.inputs[3]

                box = layout.box()

                box.column().label(text=texture.name)
                row = box.row()
                row.column().prop(tm_rot, "default_value", text="Location", )
                row.column().prop(tm_loc, "default_value", text="Rotation", )
                row.column().prop(tm_sca, "default_value", text="Scale", )

        mxs = Nodes.get('Mix Shader')
        from_node = nodes.inputs[0].links[0].from_node
        if from_node.type != 'TEX_IMAGE':
            row = box.row()
            row.label(text="Inputs:")

            if hasattr(from_node, "draw_buttons_ext"):
                from_node.draw_buttons_ext(context, box)
            elif hasattr(from_node, "draw_buttons"):
                from_node.draw_buttons(context, layout)

            value_inputs = [socket for socket in texture.inputs[1:]]
            if value_inputs:
                for socket in value_inputs:
                    row = box.row()
                    socket.draw(
                        context,
                        row,
                        tm,
                        iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context),)

class QUP_OT_swap_textures(bpy.types.Operator):
    """Swap texture  in Mix Shader"""
    bl_idname = "qup.swap_textures"
    bl_label = "Swap texture"

    user_active_node: StringProperty
    def execute(self, context):
        return {'FINISHED'}
    def invoke(self, context,event):
        nodes = bpy.context.object.active_material.node_tree.nodes
        self.user_active_node = nodes.active

        area = [area for area in bpy.context.screen.areas if area.type == "NODE_EDITOR"][0]
        with bpy.context.temp_override(area=area):

            for node in nodes:
                node.select = False

            mix_shader = nodes.get('Mix Shader')
            mix_shader.select = True
            nodes.active = mix_shader

            bpy.ops.node.nw_swap_links()

        try:
            nodes.active = self.user_active_node
            nodes.active.select = True
        except AttributeError:
            nodes.active = mix_shader

        return {'FINISHED'}

class QUP_OT_change_img_texture(bpy.types.Operator, ImportHelper):
    '''Change Image Texture'''
    bl_label = "Change Image Texture"
    bl_idname = "qup.change_img_texture"

    def execute(self, context):
        # Node_tree = bpy.context.object.active_material.node_tree
        # Nodes = Node_tree.nodes
        s_name = ('One_Texture_Shader', 'World_One_Texture_Shader')
        # node_groups = [node for node in Nodes if node.type == 'GROUP']
        node_groups = []
        for slots in context.object.material_slots:
            for nodes in slots.material.node_tree.nodes:
                if nodes.type == 'GROUP' and nodes.node_tree.name in s_name:
                    node_groups.append(nodes)
        for nodes in node_groups:
            if nodes.node_tree.name in s_name and nodes.inputs[0].links:
                img_node = nodes.inputs[0].links[0].from_node
                # bpy.context.object.active_material.node_tree.nodes.active = img_node
                image = nodes.inputs[0].links[0].from_node.image


            img = bpy.data.images.load(self.filepath)
            img_node.image = img

        # return {'FINISHED'}
        return img
class QUP_OT_draw_qup_mat(bpy.types.Operator):
    '''Handle Quick Uv Pro Materials\nShortcut: Alt-M'''
    bl_label = "Handle Quick Uv Pro Materials"
    bl_idname = "qup.draw_qup_mat"

    def execute(self, context):
        return {'FINISHED'}

    def invoke(self, context, event):
        return context.window_manager.invoke_popup(self, width=180, )


    def draw(self, context):

        Node_tree = bpy.context.object.active_material.node_tree
        Nodes = Node_tree.nodes
        node_groups = [node for node in Nodes if node.type == 'GROUP']

        slots = context.object.material_slots
        # for nodes in slots.material.node_tree.nodes:
        s_name = ('One_Texture_Shader', 'World_One_Texture_Shader')

        node_groups = []
        for slots in context.object.material_slots:
            for nodes in slots.material.node_tree.nodes:
                if nodes.type == 'GROUP' and nodes.node_tree.name in s_name and nodes.inputs[0].links:
                    node_groups.append(nodes)

        layout = self.layout
        texts = 'No Quick Uv Material Found' if not node_groups else 'Textures to Tweak'

        layout.label(text=texts, icon='GRIP')

        for nodes in node_groups:
            if nodes.node_tree.name in s_name and nodes.inputs[0].links:
                img_node = nodes.inputs[0].links[0].from_node
                image = nodes.inputs[0].links[0].from_node.image
                image = img_node.image
                image.preview_ensure()
                box = layout.box()
                box.label(text=image.name, icon_value=image.preview.icon_id, )

                # box.operator('qup.change_img_texture', text='Img Path',icon="FILE_FOLDER")


                col= box.column(align=True)
                value_inputs = [socket for socket in nodes.inputs[1:4]]
                if value_inputs:
                    for socket in value_inputs:
                        # row = box.row()
                        # box = layout.column(align=True)
                        socket.draw(
                            context,
                            col,
                            nodes,
                            iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context), )

                col.separator(factor=.5)
                value_inputs = [socket for socket in nodes.inputs[4:7]]
                if value_inputs:
                    for socket in value_inputs:
                        socket.draw(
                            context,
                            col,
                            nodes,
                            iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context), )

                col.separator(factor=.5)
                value_inputs = [socket for socket in nodes.inputs[7:8]]
                if value_inputs:
                    for socket in value_inputs:
                        socket.draw(
                            context,
                            col,
                            nodes,
                            iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context), )
                col.separator(factor=.5)
                value_inputs = [socket for socket in nodes.inputs[8:10]]
                if value_inputs:
                    for socket in value_inputs:
                        socket.draw(
                            context,
                            col,
                            nodes,
                            iface_(socket.label if socket.label else socket.name, socket.bl_rna.translation_context), )


class QUP_OT_one_texture_group(bpy.types.Operator):
    '''Add Displace Node'''
    bl_label = "Add One Texture Group"
    bl_idname = "qup.one_texture_group"

    def execute(self, context):
        mat= bpy.context.object.active_material
        n = bpy.context.object.active_material.node_tree.nodes.active
        Output_Node = mat.node_tree.nodes.get('Material Output')
        texture_node= n

        if bpy.context.object.mode == "EDIT":
            bpy.ops.object.editmode_toggle()
        # if bpy.context.object.mode != "EDIT":
        if n.type == 'TEX_IMAGE':

            file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")
            inner_path = 'NodeTree'
            nodetree_name = 'One_Texture_Shader'

            bpy.ops.wm.append(
                filepath=os.path.join(file_path, inner_path, nodetree_name),
                directory=os.path.join(file_path, inner_path),
                filename=nodetree_name
            )
            One_Shader_Node = mat.node_tree.nodes.new(type='ShaderNodeGroup')
            One_Shader_Node.node_tree = bpy.data.node_groups['One_Texture_Shader']

            One_Shader_Node.location[0] = n.location[0]+300
            One_Shader_Node.location[1] = n.location[1]
            n_link = mat.node_tree.links.new

            n_link(n.outputs[0], One_Shader_Node.inputs[0])
            n_link(One_Shader_Node.outputs[0], Output_Node.inputs['Surface'])
            n_link(One_Shader_Node.outputs[1], Output_Node.inputs['Displacement'])

        return {'FINISHED'}



class QUP_OT_warning_straight_unwrap(bpy.types.Operator):
    '''Warning Straight Unwrap'''
    bl_label = "Warning Straight Unwrap"
    bl_idname = "qup.warning_straight_unwrap"

    def execute(self, context):
        return {'FINISHED'}
    def invoke(self, context,
               event):
        return context.window_manager.invoke_popup(self, width=500,)
    def draw(self, context,):

        layout = self.layout
        layout.label(text="Warning", icon='INFO')
        layout.label(text=f"        Your Mesh or Selection has 'N_Gons' (Faces with more or less than 4 vertices).")
        layout.label(text="        Only Quads have been accurately Unwrapped and Straightened.")
        layout.label(text="        N_Gons have been removed from your selection to properly Unwrap and Straight Uvs")





obj_locked = []

def get_locked_objects():
    colls = bpy.context.scene.collection
    for obj in colls.all_objects:
        if obj.hide_select:
            obj_locked.append(obj)
        try:
            if obj not in list(bpy.data.collections['UV_GIZMO'].objects):
                obj.hide_select = True
        except KeyError:
            pass
    return {'FINISHED'}
def restore_locked_objects():
    colls = bpy.context.scene.collection
    for obj in colls.all_objects:
        obj.hide_select = False
    for obj in obj_locked:
        obj.hide_select = True
    return {'FINISHED'}

def create_qup_ghost():
    obj = bpy.context.object
    new_obj = obj.parent.copy()
    new_obj.data = obj.parent.data.copy()
    new_obj.name = 'QUP_Ghost'
    new_obj.data.name = new_obj.name
    bpy.context.collection.objects.link(new_obj)
    new_obj.parent = None
    new_obj.display_type = 'WIRE'
    if not bpy.context.scene.tool_settings.use_snap:
        new_obj.hide_viewport = True
        new_obj.select_set(False)
        new_obj.visible_camera = False
        new_obj.visible_diffuse = False

    mod_to_del = [mod for mod in new_obj.modifiers if mod.type != 'SUBSURF']
    for mod in mod_to_del:
        new_obj.modifiers.remove(mod)
    for mod in new_obj.modifiers:
        if mod.type == 'SUBSURF':
            new_obj.modifiers["Subdivision"].show_only_control_edges = False


def delete_qup_ghost():
    for obj in list(bpy.context.scene.collection.all_objects):
        if obj.name.startswith('QUP_Ghost'):
            bpy.data.objects.remove(obj)
    for block in bpy.data.meshes:
        if block.name.startswith('QUP_Ghost') and block.users == 0:
            bpy.data.meshes.remove(block)

    return {'FINISHED'}
def select_gizmo(param_gizmo):

    param_gizmo: StringProperty()
    obj = bpy.context.object

    if obj.type == 'MESH':
        Gz = None
        bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = False
        bpy.ops.object.select_all(action='DESELECT')
        objects_in_collection = bpy.data.collections["UV_GIZMO"].all_objects
        material = obj.active_material

        if param_gizmo == 'C':
            for modifier in obj.modifiers:
                if modifier.name.startswith("QuickUVProject_Cube"):
                    Gz = obj.modifiers["QuickUVProject_Cube"].projectors[0].object.parent

        if param_gizmo == 'P':
            for modifier in obj.modifiers:
                if modifier.name.startswith("QuickUVProject_Plane"):
                    Gz = obj.modifiers["QuickUVProject_Plane"].projectors[0].object

        if param_gizmo == 'W':
            Gz = material.node_tree.nodes["Texture Coordinate"].object

        Gz.select_set(True)
        bpy.context.view_layer.objects.active = Gz
        for obj in Gz.children:
            obj.hide_viewport = True
        for ob in list(objects_in_collection):
            if ob != bpy.context.object:
                ob.hide_viewport = True

    return {'FINISHED'}

Gizmo = []

objects_in_collection = []


def import_gizmo(param_gizmo):
    global Gizmo
    global objects_in_collection
    param_gizmo: StringProperty()
    obj = bpy.context.object

    file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")
    inner_path = 'Object'
    if param_gizmo == 'C':
        object_name = 'Gizmo_CUBE'
    if param_gizmo == 'P':
        object_name = 'Gizmo_PLANE'
    if param_gizmo == 'W':
        object_name = 'Gizmo_WORLD'



    bpy.ops.wm.append(
        filepath=os.path.join(file_path, inner_path, object_name),
        directory=os.path.join(file_path, inner_path),
        filename=object_name
    )

    Gizmos = [obj for obj in bpy.context.scene.objects if obj.name.startswith(object_name)]
    for Gz in Gizmos:
        # coll_from = bpy.context.scene.collection
        coll_from = bpy.context.view_layer.active_layer_collection.collection
        scene = bpy.context.scene
        link_to_name = 'UV_GIZMO'
        to_unlink = []

        try:
            link_to = scene.collection.children[link_to_name]
            try:
                link_to.objects.link(Gz)
            except RuntimeError:
                pass

        except KeyError:

            link_to = bpy.data.collections.new(link_to_name)
            scene.collection.children.link(link_to)
            bpy.context.scene.collection.children[link_to_name].id_data.color_tag = 'COLOR_03'
            link_to.objects.link(Gz)

        to_unlink.append(Gz)

        for ob in to_unlink:
            try:
                coll_from.objects.unlink(ob)
            except RuntimeError:
                pass

    bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)

    objects_in_collection = bpy.data.collections["UV_GIZMO"].all_objects
    Gizmos = [obj for obj in bpy.context.object.children if
              obj in list(objects_in_collection)]
    Gizmo = (Gizmos[-1:])

def isolate_gizmo(param_gizmo):

    objects_in_collection = bpy.data.collections["UV_GIZMO"].all_objects
    if param_gizmo == 'C':
        object_name = 'Gizmo_CUBE'
    if param_gizmo == 'P':
        object_name = 'Gizmo_PLANE'
    if param_gizmo == 'W':
        object_name = 'Gizmo_WORLD'

    for obj in bpy.context.object.children:
        if obj in list(objects_in_collection):
            Gizmos = [obj for obj in bpy.context.scene.objects if obj.name.startswith(object_name)]
            Gz = Gizmos[-1]
            Gz.select_set(True)
            bpy.context.view_layer.objects.active = Gz
            for obj in Gz.children:
                obj.hide_viewport = True

    for ob in list(objects_in_collection):
        if ob != bpy.context.object:
            ob.hide_viewport = True



def has_world_gizmo(self):
    obj = bpy.context.object
    parent = obj.parent
    tc = parent.active_material.node_tree.nodes.get("Texture Coordinate", None)
    if tc is not None and tc.object is not None and tc.object.name.startswith('Gizmo_WORLD'):
        return True


def event_gizmo(self,context, event):
    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    try:
        obj = bpy.context.object
        wm = bpy.context.window_manager
        kc = wm.keyconfigs.user
        km = kc.keymaps['3D View']
        kmi2 = km.keymap_items['Quick Uv Pro: Validate Modal']
        kmi3 = km.keymap_items['qup.gizmo_dimensions']

        if event.type in {
            'G',
            'R',
            'S',
            'C',
            'WHEELUPMOUSE',
            'WHEELDOWNMOUSE',
            'O',


        }:
            return {'PASS_THROUGH'}

        if event.type in{ 'UP_ARROW','DOWN_ARROW','A'}  and event.value == 'PRESS':
            if has_world_gizmo(self):
                bpy.ops.qup.world_gizmo_switch('INVOKE_DEFAULT')

        if event.type == 'MIDDLEMOUSE' and event.value == 'PRESS':
            override_context = bpy.context.copy()
            area = [area for area in bpy.context.screen.areas if area.type == "VIEW_3D"][0]
            override_context['window'] = bpy.context.window
            override_context['screen'] = bpy.context.screen
            override_context['area'] = area
            override_context['region'] = area.regions[-1]
            override_context['scene'] = bpy.context.scene
            override_context['space_data'] = area.spaces.active

            with context.temp_override(**override_context):
                bpy.ops.view3d.rotate('INVOKE_DEFAULT')

        if event.type == 'T' and event.value == 'PRESS':
            bpy.ops.qup.shortcut_popup('INVOKE_DEFAULT')

        if event.alt and event.type == 'RIGHTMOUSE'and event.value == 'PRESS':
            bpy.ops.view3d.cursor3d('INVOKE_DEFAULT')

        if event.shift and event.type == 'TAB' and event.value == 'PRESS':
            context.scene.tool_settings.use_snap = not context.scene.tool_settings.use_snap
            for obj in context.collection.all_objects:
                if obj.name.startswith('QUP_Ghost'):
                    if context.scene.tool_settings.use_snap:
                        not obj.hide_viewport
                    obj.hide_viewport = not obj.hide_viewport

        if event.type == kmi3.type and event.value == 'PRESS':
            bpy.ops.qup.gizmo_dimensions('INVOKE_DEFAULT')

        if event.ctrl and event.type == 'Z' and event.value == 'PRESS':
            self.report({'WARNING'}, "No Undo While Modal Gizmo is Running")    

        if event.alt and event.type == 'Z' and event.value == 'PRESS':
            bpy.ops.wm.call_menu_pie(name="VIEW3D_MT_shading_pie")

        if event.type == 'X' and event.value == 'PRESS':
            x = obj.scale[0]
            obj.scale[0] = -x

        if event.shift and event.type == 'X' and event.value == 'PRESS':
            y = obj.scale[1]
            obj.scale[1] = -y

        if event.ctrl and event.type == 'X' and event.value == 'PRESS':
            z = obj.scale[2]
            obj.scale[2] = -z


        if event.type == 'NUMPAD_0' and event.value == 'PRESS':
            bpy.ops.qup.fit_gizmo_to_mesh()


        if event.ctrl and event.type == 'NUMPAD_0' and event.value == 'PRESS':
            bpy.ops.qup.fit_gizmo_selected_faces('INVOKE_DEFAULT')
            self.user_sca = obj.scale
            self.user_rot = obj.rotation_euler
            self.user_loc = obj.location

        for i in (1, 2, 3, 4, 5, 6, 7, 8, 9):
            if event.type == f'NUMPAD_{i}' and event.value == 'PRESS':
                obj.dimensions = [i, i, i]


        if event.type in {'RET', 'NUMPAD_ENTER'} or event.type == kmi2.type and event.value == 'PRESS':
            delete_qup_ghost()
            print('del ghost ok')

            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color
            bpy.context.view_layer.objects.active = self.user_active_object
            self.user_active_object.select_set(True)
            bpy.context.scene.tool_settings.use_proportional_edit_objects = self.user_proportional

            print('user state back ok')
            # points.clear()

            all_objects = bpy.context.scene.collection.children["UV_GIZMO"].all_objects[:]
            for obj in all_objects:
                obj.hide_viewport = False

            print('hide collection ok')
            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                print('remove handlers ok')
            except ValueError:
                pass
            return {'FINISHED'}

    except Exception as e:
        context.preferences.themes[0].view_3d.object_active = self.user_active_color
        raise e
    return {'RUNNING_MODAL'}

def has_uv_gizmo(self):
    obj = context.object
    mc = [modifier for modifier in obj.modifiers if modifier.name.startswith("QuickUVProject_Cube")]
    mp = [modifier for modifier in obj.modifiers if modifier.name.startswith("QuickUVProject_Plane")]
    G = [obj for obj in bpy.context.object.children if obj.name.startswith("Gizmo_WORLD")]

    if len(mc) > 0 or len(mp) > 0 or len(G) > 0:
        return True


def error_gizmo(self):
    obj = bpy.context.object
    mc = [modifier for modifier in obj.modifiers if modifier.name.startswith("QuickUVProject_Cube")]
    mp = [modifier for modifier in obj.modifiers if modifier.name.startswith("QuickUVProject_Plane")]
    # G = [obj for obj in bpy.context.object.children if obj.name.startswith("Gizmo_WORLD")]

    if len(mc) > 0 or len(mp) > 0:
        self.report({'ERROR'}, " !!! This Mesh has a Uv Gizmo. Uv Handlers won't work"
                               " \n Use Gizmo Instead or Apply it and then use Handlers")
        return {'FINISHED'}


def undo_gizmo(self):

    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_dim: FloatVectorProperty()

    obj = bpy.context.object
    #
    if bpy.context.active_operator.name == 'Rotate':
        obj.rotation_euler = self.user_rot
    if bpy.context.active_operator.name == 'Move':
        obj.location = self.user_loc
    if bpy.context.active_operator.name == 'Resize':
        obj.scale = self.user_sca

# sizer = []
class QUP_OT_gizmo_lock_ratio(bpy.types.Operator):
    """Lock Gizmo Ratio based on Image Texture Ratio"""
    bl_label = "Lock Ratio Texture"
    bl_idname = "qup.gizmo_lock_ratio"


    def execute(self, context):

        obj = bpy.context.object
        Node_tree = obj.parent.active_material.node_tree
        Nodes = Node_tree.nodes

        for n in Nodes:
            if n.type == 'TEX_IMAGE':
                size_img = n.image.size

        ratio = size_img[0]/size_img[1]
        x = obj.dimensions[0]
        y = obj.dimensions[1]

        obj.dimensions[1] = x / ratio

        return {'FINISHED'}


class QUP_OT_gizmo_dimensions(bpy.types.Operator):
    """Set Dimensions"""
    bl_label = "Gizmo Real Dimension"
    bl_idname = "qup.gizmo_dimensions"

    lock_ratio: bpy.props.BoolProperty()

    def execute(self, context):

        return {'FINISHED'}

    def invoke(self, context, event):
        if bpy.context.object:
            return context.window_manager.invoke_popup(self, width=150)
        else:
            return {'FINISHED'}

    def draw(self, context):

        obj = bpy.context.object
        layout = self.layout

        layout.label(text='Gizmo settings', icon='GRIP')
        box = layout.box()
        row = box.row()
        row.column().prop(obj, "dimensions",text='Dimensions')
        if obj.name.startswith("Gizmo_PLANE"):
            box.column().operator("qup.gizmo_lock_ratio",text='Keep Texture Ratio', icon='NONE')

        box = layout.box()
        row = box.row()
        row.column().prop(obj, "rotation_euler", text='Rotation')

class QUP_OT_fit_gizmo_to_mesh(bpy.types.Operator):
    """Fit Gizmo to Msh"""
    bl_idname = "qup.fit_gizmo_to_mesh"
    bl_label = "Fit Gizmo To Mesh"
    bl_options = {'UNDO', 'REGISTER'}

    def execute(self, context):

        obj = bpy.context.object
        objp = bpy.context.object.parent

        all_points = []

        bound_box = objp.bound_box
        bound_box_points = []
        world_mat = objp.matrix_world
        # amx = bpy.data.objects[objp.name].matrix_world
        aloc, arot, asca = world_mat.decompose()
        arotx, aroty, arotz = arot.to_euler('XYZ')

        for bound_box_point in bound_box:
            bound_box_points.append(
                world_mat @ Vector((bound_box_point[0], bound_box_point[1], bound_box_point[2])))

            all_points.extend(bound_box_points)

        if len(all_points):
            x, y, z = all_points[0]
            min_x = x
            max_x = x
            min_y = y
            max_y = y
            min_z = z
            max_z = z

            for i in range(1, len(all_points)):
                x, y, z = all_points[i]
                min_x = min(x, min_x)
                max_x = max(x, max_x)
                min_y = min(y, min_y)
                max_y = max(y, max_y)
                min_z = min(z, min_z)
                max_z = max(z, max_z)

        av_x = (max_x + min_x) / float(2)
        av_y = (max_y + min_y) / float(2)
        av_z = (max_z + min_z) / float(2)
        location_p = (av_x, av_y, av_z)

        obj.dimensions = objp.dimensions
        obj.rotation_euler = arot.to_euler('XYZ')
        obj.location = location_p

        return {'FINISHED'}

points = []
class QUP_OT_fit_gizmo_selected_faces(bpy.types.Operator):
    """Fit Gizmo to Msh"""
    bl_idname = "qup.fit_gizmo_selected_faces"
    bl_label = "Fit Gizmo To Selected Faces"
    bl_options = {'UNDO', 'REGISTER'}

    # points = []
    def execute(self, context):

        obj = bpy.context.object
        objp = bpy.context.object.parent
        mesh = objp.data

        selected_verts = [v for v in mesh.vertices if v.select]

        points = []

        for v in selected_verts:
            if (v.select == True):
                points.append(v.co)
                coordinates = points
        def get_selected_vertices(mesh):
            selected_vertices = []
            for v in mesh.vertices:
                if v.select:
                    selected_vertices.append(v.co)
            return selected_vertices
        def dimensions(selected_vertices):
            x = [c[0] for c in coordinates]
            y = [c[1] for c in coordinates]
            z = [c[2] for c in coordinates]

            x_min = min(x)
            x_max = max(x)
            y_min = min(y)
            y_max = max(y)
            z_min = min(z)
            z_max = max(z)

            minimum = Vector((x_min, y_min, z_min))
            maximum = Vector((x_max, y_max, z_max))

            dimensions = maximum - minimum

            return dimensions

        def calculate_bounding_box_center(selected_vertices):
            min_x = min(selected_vertices, key=lambda v: v.x).x
            max_x = max(selected_vertices, key=lambda v: v.x).x
            min_y = min(selected_vertices, key=lambda v: v.y).y
            max_y = max(selected_vertices, key=lambda v: v.y).y
            min_z = min(selected_vertices, key=lambda v: v.z).z
            max_z = max(selected_vertices, key=lambda v: v.z).z

            center = mathutils.Vector((
                (max_x + min_x) / 2,
                (max_y + min_y) / 2,
                (max_z + min_z) / 2
            ))
            return center

        selected_vertices = get_selected_vertices(mesh)

        # center = calculate_bounding_box_center(selected_vertices)

        obj.location = objp.matrix_world @ calculate_bounding_box_center(selected_vertices)
        # obj.dimensions = dimensions(coordinates)
        obj.dimensions = dimensions(selected_vertices)


        return {'FINISHED'}
class QUP_OT_add_world_material(bpy.types.Operator, ImportHelper):
    """Create One texture Shader"""
    bl_idname = "qup.add_world_material"
    bl_label = "add mat world"

    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()
    user_active_object: StringProperty
    user_object_mode: StringProperty
    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_proportional: BoolProperty()


    def modal(self, context, event):
        obj = context.object
        if event.type in {'ESC'}:

            bpy.data.objects.remove(obj)
            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            bpy.context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color

            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            except ValueError:
                pass
            return {'CANCELLED'}

        return event_gizmo(self, context, event)

    def execute(self, context):
    # def invoke(self, context,event):
        obj = context.object
        mesh=obj.data

        self.user_active_object = obj
        self.user_active_color = context.preferences.themes[0].view_3d.object_active
        self.user_wire_color = context.preferences.themes[0].view_3d.wire
        self.user_proportional = bpy.context.scene.tool_settings.use_proportional_edit_objects
        self.user_object_mode = bpy.context.object.mode

        bpy.context.scene.tool_settings.use_proportional_edit_objects = False

        head, tail = os.path.split(self.filepath)

        try:
            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = False
            for obj in bpy.context.scene.collection.children["UV_GIZMO"].objects:
                obj.hide_viewport = True
        except KeyError:
            pass

        if bpy.context.object.mode == "EDIT":
            bpy.ops.object.editmode_toggle()
            bpy.ops.object.material_slot_add()

        try:

            ######## APPEND World_One_Texture_Shader Node Group ############################

            file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")

            inner_path = 'NodeTree'
            nodetree_name = 'World_One_Texture_Shader'

            bpy.ops.wm.append(
                filepath=os.path.join(file_path, inner_path, nodetree_name),
                directory=os.path.join(file_path, inner_path),
                filename=nodetree_name
            )

            ######## Create New PRINCPLED BSDF ############################

            material_bascic = bpy.data.materials.new(name="New Mat")
            material_bascic.use_nodes = True

            bpy.context.object.active_material = material_bascic

            node_to_delete = material_bascic.node_tree.nodes['Principled BSDF']
            material_bascic.node_tree.nodes.remove(node_to_delete)

            Output_Node = material_bascic.node_tree.nodes.get('Material Output')

            One_Shader_Node = material_bascic.node_tree.nodes.new(type='ShaderNodeGroup')

            One_Shader_Node.node_tree = bpy.data.node_groups['World_One_Texture_Shader']
            One_Shader_Node.name = 'World_One_Texture_Shader'
            One_Shader_Node.label = 'World_One_Texture_Shader'


            One_Shader_Node.location = (-300, 215)

            Texture_Node = material_bascic.node_tree.nodes.new('ShaderNodeTexImage')

            material_bascic.node_tree.nodes["Image Texture"].projection = 'BOX'
            material_bascic.node_tree.nodes["Image Texture"].projection_blend = 0.25

            Texture_Node.location = (-600, 215)

            Mapping_Node = material_bascic.node_tree.nodes.new('ShaderNodeMapping')
            i = (0, 1, 2)
            value = .5
            for i in i:
                Mapping_Node.inputs[1].default_value[i] = value
                Mapping_Node.inputs[3].default_value[i] = value

            Mapping_Node.location = (-900, 215)

            Coord_Node = material_bascic.node_tree.nodes.new('ShaderNodeTexCoord')
            Coord_Node.location = (-1200, 215)

            material_bascic.node_tree.links.new(Texture_Node.outputs[0], One_Shader_Node.inputs[0])
            material_bascic.node_tree.links.new(Texture_Node.outputs[1], One_Shader_Node.inputs[7])
            material_bascic.node_tree.links.new(One_Shader_Node.outputs[0], Output_Node.inputs['Surface'])

            material_bascic.node_tree.links.new(Coord_Node.outputs[3], Mapping_Node.inputs[0])
            material_bascic.node_tree.links.new(Mapping_Node.outputs[0], Texture_Node.inputs[0])

            nodes = material_bascic.node_tree.nodes
            for node in nodes:
                node.select = False

            Texture_Node.select = True
            Output_Node.select = True
            nodes.active = Texture_Node

            img = bpy.data.images.load(self.filepath)
            Texture_Node.image = img

            # if Texture_Node.type == 'TEX_IMAGE':
            #     Gz.name = f'{Gz.name}_{Texture_Node.image.name[:-4]}'
            # else:
            #     Gz.name = f'{Gz.name}_{node.name}'


            my_object = bpy.context.view_layer.objects.active
            my_object.hide_viewport = False
            my_object.select_set(True)

            material_bascic.name = tail[:-4]
            # material_bascic.name = tail

            bpy.ops.object.editmode_toggle()
            bpy.ops.object.material_slot_assign()
            bpy.ops.object.editmode_toggle()

            import_gizmo(param_gizmo='W')

            Gizmos = [obj for obj in bpy.context.scene.objects if obj.name.startswith("Gizmo_WORLD")]
            Gizmo = (Gizmos[-1:])

            for Gz in Gizmo:
                my_object = bpy.context.view_layer.objects.active  # assign variable
                my_object.select_set(True)  # select the object

                bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)
                bpy.ops.object.select_all(action='DESELECT')
                bpy.context.view_layer.objects.active = bpy.data.objects[Gz.name]
                my_object = bpy.context.view_layer.objects.active  # assign variable
                material_bascic.node_tree.nodes["Texture Coordinate"].object = my_object

                if Texture_Node.type == 'TEX_IMAGE':
                    Gz.name = f'{Gz.name}_{Texture_Node.image.name[:-4]}'
                else:
                    Gz.name = f'{Gz.name}_{node.name}'


                my_object.select_set(True)

            Color = bpy.context.preferences.addons[__package__].preferences.Color_World_Gizmo
            bpy.context.preferences.themes[0].view_3d.object_active = (Color[0], Color[1], Color[2])
            #
            if self.user_object_mode == "OBJECT":
                bpy.ops.qup.fit_gizmo_to_mesh('INVOKE_DEFAULT')
            if self.user_object_mode == "EDIT":
                bpy.ops.qup.fit_gizmo_selected_faces('INVOKE_DEFAULT')

            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_gizmo_world_text, args, 'WINDOW', 'POST_PIXEL')

            context.window_manager.modal_handler_add(self)

            # bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
            # obj = context.object


            # self.user_sca = obj.scale
            # self.user_rot = obj.rotation_euler
            # self.user_loc = obj.location

        except Exception as e:
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            raise e

        return {'RUNNING_MODAL'}


class QUP_OT_add_gizmo_world(bpy.types.Operator):
    '''Add World Gizmo Linked to Active Image Texture to Scale & Rotate UVs '''

    bl_label = "add World Gizmo"
    bl_idname = "qup.add_gizmo_world"

    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()
    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_proportional: BoolProperty()

    def modal(self, context, event):
        obj = context.object
        if event.type in {'ESC'}:
            bpy.data.objects.remove(obj)
            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color

            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            except ValueError:
                pass
            return {'CANCELLED'}

        return event_gizmo(self, context, event)

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event):
        obj = context.object
        self.user_active_object = obj
        self.user_active_color = context.preferences.themes[0].view_3d.object_active
        self.user_wire_color = context.preferences.themes[0].view_3d.wire
        self.user_proportional = bpy.context.scene.tool_settings.use_proportional_edit_objects

        bpy.context.scene.tool_settings.use_proportional_edit_objects = False


        try:
            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = False
        except KeyError:
            pass

        if bpy.context.object.mode != "OBJECT":
            bpy.ops.object.editmode_toggle()

        if bpy.context.object.mode == "OBJECT":
            import_gizmo(param_gizmo='W')

            Gizmos = [obj for obj in bpy.context.scene.objects if obj.name.startswith("Gizmo_WORLD")]
            Gizmo = (Gizmos[-1:])

            for Gz in Gizmo:
                mat = context.object.active_material.node_tree
                mat_nodes = mat.nodes
                node = mat.nodes.active


                my_object = bpy.context.view_layer.objects.active  # assign variable
                my_object.select_set(True)  # select the object

                bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)
                bpy.ops.object.select_all(action='DESELECT')
                bpy.context.view_layer.objects.active = bpy.data.objects[Gz.name]
                my_object = bpy.context.view_layer.objects.active
                my_object.select_set(True)

                # for node in mat_nodes:
                Texture_Node = mat_nodes.active
                if node.type == 'TEX_IMAGE':

                    Texture_Node.projection = 'BOX'
                    Texture_Node.projection_blend = .25

                loc = Texture_Node.location

                Mapping_Node = mat_nodes.new('ShaderNodeMapping')
                Mapping_Node.location = (loc[0]-200,loc[1])

                Mapping_Node.inputs[1].default_value = (.5,.5,.5)
                Mapping_Node.inputs[3].default_value = (.5,.5,.5)

                Coord_Node = mat_nodes.new('ShaderNodeTexCoord')
                Coord_Node.location = (loc[0]-400, loc[1])

                mat.links.new(Coord_Node.outputs[3], Mapping_Node.inputs[0])
                mat.links.new(Mapping_Node.outputs[0], Texture_Node.inputs[0])

                Coord_Node.object = my_object

                if node.type == 'TEX_IMAGE':
                    Gz.name = f'{Gz.name}_{Texture_Node.image.name[:-4]}'
                else:
                    Gz.name = f'{Gz.name}_{node.name}'

            Color = bpy.context.preferences.addons[__package__].preferences.Color_World_Gizmo
            bpy.context.preferences.themes[0].view_3d.object_active = (Color[0], Color[1], Color[2])

            isolate_gizmo('W')
            bpy.ops.qup.fit_gizmo_to_mesh('INVOKE_DEFAULT')

            # obj.name = f'{obj.name}+{Texture_Node.image.name}'
            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_gizmo_world_text, args, 'WINDOW',
                                                                  'POST_PIXEL')

            context.window_manager.modal_handler_add(self)

            # bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
            self.user_sca = obj.scale
            self.user_rot = obj.rotation_euler
            self.user_loc = obj.location

            # return {'RUNNING_MODAL'}

        return {'RUNNING_MODAL'}

class QUP_OT_gizmo_cube(bpy.types.Operator):
    '''"Create Uv Cube Gizmo"'''

    bl_label = "UvGizmoCube"
    bl_idname = "qup.gizmo_cube"
    # bl_options = {'UNDO', 'REGISTER'}

    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()
    user_active_object: StringProperty

    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_proportional: BoolProperty()

    def create_empty(self, name, display_type, display_size, parent):
        empty = bpy.data.objects.new(name, None)
        empty.empty_display_type = display_type
        empty.empty_display_size = display_size
        empty.parent = parent
        empty.show_in_front = True

        for col in parent.users_collection:
            if col not in empty.users_collection:
                col.objects.link(empty)

        return empty

    def modal(self, context, event):

        obj = context.object

        if event.type in {'ESC'}:
            delete_qup_ghost()
            objp = obj.parent
            objp.modifiers.remove(objp.modifiers.get("QuickUVProject_Cube"))
            objects_in_collection = bpy.data.collections["UV_GIZMO"].all_objects
            for objs in bpy.context.object.children_recursive:
                if objs in list(objects_in_collection):
                    bpy.data.objects.remove(objs)
            bpy.data.objects.remove(obj)

            for obj in bpy.context.scene.collection.children["UV_GIZMO"].all_objects:
                obj.hide_viewport = False

            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            bpy.context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color

            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            except ValueError:
                pass
            return {'CANCELLED'}

        return event_gizmo(self, context, event)

    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event):
    # def execute(self, context,):

        obj = bpy.context.object
        self.user_active_object = obj
        self.user_active_color = context.preferences.themes[0].view_3d.object_active
        self.user_wire_color = context.preferences.themes[0].view_3d.wire
        self.user_proportional = bpy.context.scene.tool_settings.use_proportional_edit_objects

        bpy.context.scene.tool_settings.use_proportional_edit_objects = False
        try:
            try:
                bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = False
            except KeyError:
                pass

            if obj.mode == "EDIT":
                bpy.ops.object.editmode_toggle()

            import_gizmo(param_gizmo='C')
            bpy.ops.object.parent_set(type='OBJECT', keep_transform=False)


            mod = obj.modifiers.new(name='QuickUVProject_Cube', type='UV_PROJECT')

            mod.projector_count = 6

            for obj in bpy.context.selected_objects:
                if obj.name.startswith('Gizmo_CUBE'):
                    bpy.context.view_layer.objects.active = obj

            my_object = bpy.context.view_layer.objects.active
            my_object.select_set(True)


            size = .5
            con = my_object

            xp = self.create_empty('x+', 'SINGLE_ARROW', size, con)
            xn = self.create_empty('x-', 'SINGLE_ARROW', size, con)
            yp = self.create_empty('y+', 'SINGLE_ARROW', size, con)
            yn = self.create_empty('y-', 'SINGLE_ARROW', size, con)
            zp = self.create_empty('z+', 'SINGLE_ARROW', size, con)
            zn = self.create_empty('z-', 'SINGLE_ARROW', size, con)

            xp.rotation_euler = [math.radians(90), 0, math.radians(90)]
            xn.rotation_euler = [math.radians(90), 0, math.radians(-90)]
            yp.rotation_euler = [math.radians(-90), math.radians(180), 0]
            yn.rotation_euler = [math.radians(90), 0, 0]
            zp.rotation_euler = [0, 0, 0]
            zn.rotation_euler = [0, math.radians(180), 0]

            for index, empty in enumerate([xp, xn, yp, yn, zp, zn]):
                mod.projectors[index].object = empty

            BLUE = bpy.context.preferences.addons[__package__].preferences.Text_Color_Modal_Uv
            context.preferences.themes[0].view_3d.object_active = (BLUE[0], BLUE[1] - .3, BLUE[2])
            context.preferences.themes[0].view_3d.wire = (BLUE[0] + .5, BLUE[1], BLUE[2] - .1)

            isolate_gizmo('C')
            create_qup_ghost()

            bpy.ops.qup.fit_gizmo_to_mesh('INVOKE_DEFAULT')

            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_gizmo_uv_text, args, 'WINDOW',
                                                                      'POST_PIXEL')

            context.window_manager.modal_handler_add(self)

            bpy.ops.transform.resize(value=(1, 1, 1), )
            bpy.ops.transform.rotate(value=0, )
            bpy.ops.transform.translate(value=(0, 0, 0),)
            bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

            self.user_sca = obj.scale
            self.user_rot = obj.rotation_euler
            self.user_loc = obj.location

        except Exception as e:
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            raise e

        return {'RUNNING_MODAL'}

class QUP_OT_gizmo_plane(bpy.types.Operator):
    '''"Create Uv Plane Gizmo"'''
    bl_label = "UvGizmoRealSize"
    bl_idname = "qup.gizmo_plane"
    bl_options = {'UNDO', 'REGISTER'}

    scale: bpy.props.IntVectorProperty()
    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()

    user_active_object: StringProperty

    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_proportional: BoolProperty()

    def modal(self, context, event):

        obj = context.object

        if event.type in {'ESC'}:

            objp = obj.parent
            objp.modifiers.remove(objp.modifiers.get("QuickUVProject_Plane"))
            bpy.data.objects.remove(obj)

            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            bpy.context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color

            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            except ValueError:
                pass

            return {'CANCELLED'}
        #
        return event_gizmo(self, context, event)


    @classmethod
    def poll(cls, context):
        return context.active_object is not None

    def invoke(self, context, event):

        self.user_active_color = bpy.context.preferences.themes[0].view_3d.object_active
        self.user_wire_color = context.preferences.themes[0].view_3d.wire
        obj = bpy.context.object
        self.user_active_object = obj
        self.user_proportional = bpy.context.scene.tool_settings.use_proportional_edit_objects

        bpy.context.scene.tool_settings.use_proportional_edit_objects = False

        try:
            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = False
        except KeyError:
            pass

        if obj.mode == "EDIT":
            bpy.ops.object.editmode_toggle()

        import_gizmo(param_gizmo='P')


        mod = bpy.context.active_object.modifiers.new(type='UV_PROJECT', name="QuickUVProject_Plane")

        for obj in bpy.context.selected_objects:
            if obj.name.startswith('Gizmo_PLANE'):
                mod.projectors[0].object = bpy.data.objects[obj.name]
                bpy.context.view_layer.objects.active = obj
                # bpy.ops.transform.resize(value=(.5, .5, .5), )

        my_object = bpy.context.view_layer.objects.active
        my_object.select_set(True)

        BLUE = bpy.context.preferences.addons[__package__].preferences.Text_Color_Modal_Uv
        bpy.context.preferences.themes[0].view_3d.object_active = (BLUE[0], BLUE[1] - .3, BLUE[2])
        context.preferences.themes[0].view_3d.wire = (BLUE[0] + .5, BLUE[1], BLUE[2] - .1)

        isolate_gizmo('P')
        create_qup_ghost()

        bpy.ops.qup.fit_gizmo_to_mesh('INVOKE_DEFAULT')
        # bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

        args = (self, context)
        self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_gizmo_uv_text, args, 'WINDOW',
                                                              'POST_PIXEL')
        context.window_manager.modal_handler_add(self)

        bpy.ops.transform.resize(value=(1, 1, 1), )
        bpy.ops.transform.rotate(value=0, )
        bpy.ops.transform.translate(value=(0, 0, 0), )
        # bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)

        self.user_sca = obj.scale
        self.user_rot = obj.rotation_euler
        self.user_loc = obj.location

        return {'RUNNING_MODAL'}


class QUP_OT_select_world_gizmo(bpy.types.Operator):
    """Jump to Gizmo """
    bl_idname = "qup.select_world_gizmo"
    bl_label = "Quick Uv Pro: Select World Gizmo"

    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()
    user_active_object: StringProperty
    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_proportional: BoolProperty()

    def modal(self, context, event):

        if event.type in {'ESC'}:
            delete_qup_ghost()
            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            bpy.context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color

            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            except ValueError:
                pass
            return {'CANCELLED'}

        return event_gizmo(self, context, event)

    def invoke(self, context, event):

        try:
            obj = context.object
            self.user_active_object = obj
            self.user_active_color = context.preferences.themes[0].view_3d.object_active
            self.user_wire_color = context.preferences.themes[0].view_3d.wire
            self.user_proportional = bpy.context.scene.tool_settings.use_proportional_edit_objects

            # self.user_sca = obj.scale
            # self.user_rot = obj.rotation_euler
            # self.user_loc = obj.location

            bpy.context.scene.tool_settings.use_proportional_edit_objects = False

            # bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
            # self.user_sca = obj.scale
            # self.user_rot = obj.rotation_euler
            # self.user_loc = obj.location

            select_gizmo(param_gizmo='W')

            create_qup_ghost()

            Color = bpy.context.preferences.addons[__package__].preferences.Color_World_Gizmo
            context.preferences.themes[0].view_3d.object_active = (Color[0], Color[1], Color[2])
            context.preferences.themes[0].view_3d.wire = (Color[0] + .5, Color[1]+.2, Color[2] +.4)

            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_gizmo_world_text, args, 'WINDOW', 'POST_PIXEL')
            context.window_manager.modal_handler_add(self)

            obj = bpy.context.object
            objp = obj.parent
            mesh = objp.data
            mesh.update()

            self.user_sca = obj.scale
            self.user_rot = obj.rotation_euler
            self.user_loc = obj.location

        except Exception as e:
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            raise e

        return {'RUNNING_MODAL'}

class QUP_OT_select_cube_gizmo(bpy.types.Operator):
    """Jump to Cube Gizmo """
    bl_idname = "qup.select_cube_gizmo"
    bl_label = "Quick Uv Pro: Select Cube Gizmo"

    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()
    user_active_object: StringProperty
    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_proportional: BoolProperty()
    # user_snap: BoolProperty()

    def modal(self, context, event):

        if event.type in {'ESC'}:
            delete_qup_ghost()
            context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color
            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            except ValueError:
                pass
            return {'CANCELLED'}

        return event_gizmo(self, context, event)

    def invoke(self, context, event):

        try:
            obj = context.object
            self.user_active_object = obj
            self.user_active_color = context.preferences.themes[0].view_3d.object_active
            self.user_wire_color = context.preferences.themes[0].view_3d.wire
            self.user_proportional = bpy.context.scene.tool_settings.use_proportional_edit_objects
            # self.user_snap = context.scene.tool_settings.use_snap
            bpy.context.scene.tool_settings.use_proportional_edit_objects = False
            try:
                select_gizmo(param_gizmo='C')

                create_qup_ghost()


                BLUE = bpy.context.preferences.addons[__package__].preferences.Text_Color_Modal_Uv

                context.preferences.themes[0].view_3d.object_active = (BLUE[0], BLUE[1] - .3, BLUE[2])
                context.preferences.themes[0].view_3d.wire = (BLUE[0]+.5, BLUE[1], BLUE[2]-.1)

                args = (self, context)
                self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_gizmo_uv_text, args, 'WINDOW', 'POST_PIXEL')
                context.window_manager.modal_handler_add(self)

                bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
                self.user_sca = obj.scale
                self.user_rot = obj.rotation_euler
                self.user_loc = obj.location

            except AttributeError:
                self.report({'ERROR'}, "You have Manually deleted the Gizmo!"
                                       "\n"
                                       "\nGizmo Modifier has been remove to clean the Modifiers Stack"
                                       "\nUse Apply button (Down Arrow) instead of deleting Gizmos"
                                       "\nYou can Now Recreate a new One")

                mod_to_remove = obj.modifiers.get("QuickUVProject_Cube")
                for i in [0,1,2,3,4,5]:
                    emp = obj.modifiers['QuickUVProject_Cube'].projectors[i].object
                    bpy.data.objects.remove(emp)

                obj.modifiers.remove(mod_to_remove)

                return {'CANCELLED'}
        except Exception as e:
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            raise e
        return {'RUNNING_MODAL'}


class QUP_OT_select_plane_gizmo(bpy.types.Operator):
    """Jump to Plane Gizmo """
    bl_idname = "qup.select_plane_gizmo"
    bl_label = "Quick Uv Pro: Select Plane Gizmo"

    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()
    user_active_object: StringProperty
    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()

    user_proportional: BoolProperty()
    def modal(self, context, event):

        if event.type in {'ESC'}:
            delete_qup_ghost()
            bpy.context.view_layer.layer_collection.children["UV_GIZMO"].exclude = True
            bpy.context.preferences.themes[0].view_3d.object_active = self.user_active_color
            context.preferences.themes[0].view_3d.wire = self.user_wire_color

            try:
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            except ValueError:
                pass

            return {'CANCELLED'}

        return event_gizmo(self, context, event)

    def invoke(self, context, event):
        global obj_locked
        try:
            obj = context.object
            self.user_active_object = obj
            self.user_active_color = context.preferences.themes[0].view_3d.object_active
            self.user_wire_color = context.preferences.themes[0].view_3d.wire
            self.user_proportional = bpy.context.scene.tool_settings.use_proportional_edit_objects

            bpy.context.scene.tool_settings.use_proportional_edit_objects = False
            try:
                select_gizmo(param_gizmo='P')
                create_qup_ghost()

                BLUE = bpy.context.preferences.addons[__package__].preferences.Text_Color_Modal_Uv

                context.preferences.themes[0].view_3d.object_active = (BLUE[0], BLUE[1] - .3, BLUE[2])
                context.preferences.themes[0].view_3d.wire = (BLUE[0] + .5, BLUE[1], BLUE[2] - .1)

                args = (self, context)
                self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_gizmo_uv_text, args, 'WINDOW', 'POST_PIXEL')
                context.window_manager.modal_handler_add(self)

                # bpy.ops.object.transform_apply(location=False, rotation=False, scale=True)
                self.user_sca = obj.scale
                self.user_rot = obj.rotation_euler
                self.user_loc = obj.location


            except AttributeError:
                self.report({'ERROR'}, "You have Manually deleted the Gizmo!"
                                       "\n"
                                       "\nGizmo Modifier has been remove to clean the Modifiers Stack"
                                       "\nUse Apply button (Down Arrow) instead of deleting Gizmos"
                                       "\nYou can Now Recreate a new One")

                mod_to_remove = obj.modifiers.get("QuickUVProject_Plane")
                obj.modifiers.remove(mod_to_remove)

                return {'CANCELLED'}
        except Exception as e:
            context.preferences.themes[0].view_3d.object_active = self.user_active_color

            raise e
        return {'RUNNING_MODAL'}
class QUP_OT_displace_map(bpy.types.Operator, ImportHelper):
    bl_label = "Add Displacement Map"
    bl_idname = "qup.displace_map"

    first_value: FloatProperty()
    def execute(self, _context):
        # filename, extension = os.path.splitext(self.filepath)
        # head, tail = os.path.split(self.filepath)
        # print('Selected:', self.filepath)
        # print('File name:', filename)
        # print('File extension:', extension)
        # print('Name:', tail)

        obj = bpy.context.view_layer.objects.active

        img = bpy.data.images.load(self.filepath)
        sub = obj.modifiers.new(name= "Subdivision",type='SUBSURF')
        sub.levels = 3
        disp_mod = obj.modifiers.new(name= "Displace_map",type='DISPLACE')
        disp_mod.strength = .1
        disp_map = bpy.data.textures.new(name="Displace Map", type='IMAGE')
        bpy.context.object.modifiers["Displace_map"].texture_coords = 'UV'
        bpy.ops.mesh.uv_texture_add()
        bpy.context.object.data.uv_layers["UVMap.001"].name = "UVMap_Displace"
        bpy.context.object.data.uv_layers["UVMap_Displace"].active_render = True
        bpy.context.object.modifiers["Displace_map"].uv_layer = "UVMap_Displace"
        bpy.context.object.modifiers["Displace_map"].show_on_cage = True
        bpy.context.object.modifiers["Displace_map"].show_in_editmode = True

        disp_map.image = img
        disp_mod.texture = disp_map

        return {'FINISHED'}

class QUP_OT_displace_modal(bpy.types.Operator):
    """Launch Modal for Displace"""
    bl_idname = "qup.displace_modal"
    bl_label = "Modal Displace Qup"

    first_value : FloatProperty()
    first_value_blur : FloatProperty()
    user_active_color : FloatVectorProperty()
    def modal(self, context, event):
        pass_through_events = {
            'MIDDLEMOUSE','LEFTMOUSE', 'BACK_SPACE', 'TAB'
            'Q', 'I', 'U', 'F', 'E', 'A', 'K', 'C', 'W','P','S'
            # 'B',
            # 'WHEELDOWNMOUSE',
            # 'WHEELUPMOUSE',
        }
        try:
            if event.type in pass_through_events:
                return {'PASS_THROUGH'}
            # if event.type == 'S' and event.value ='PRESS':
            if event.type == 'B' and event.value == 'PRESS':
                bpy.ops.wm.call_menu_pie(name="QUP_MT_pie_uv_pro")
            if event.type == 'WHEELUPMOUSE':
                if event.ctrl:
                    bpy.context.object.modifiers["Displace_map"].texture.filter_size += 5
                elif event.shift:
                    bpy.context.object.modifiers["Displace_map"].strength += .01
                else:
                    bpy.context.object.modifiers["Displace_map"].strength += .1

            if event.type == 'WHEELDOWNMOUSE':
                if event.ctrl:
                    bpy.context.object.modifiers["Displace_map"].texture.filter_size -= 5
                elif event.shift:
                    bpy.context.object.modifiers["Displace_map"].strength -= .01
                else:
                    bpy.context.object.modifiers["Displace_map"].strength -= .1


            if event.ctrl and event.type == 'MIDDLEMOUSE':
                bpy.ops.qup.modal_uv("INVOKE_DEFAULT")



            elif event.type in {'RET', 'BUTTON4MOUSE'}:
                # bpy.context.object.data.uv_layers["UVMap_Displace"].active_render = False
                obj=bpy.context.object
                if obj.mode == "EDIT":
                    bpy.ops.object.editmode_toggle()
                bpy.context.object.data.uv_layers["UVMap"].active = True
                bpy.context.object.data.uv_layers["UVMap"].active_render = True
                context.preferences.themes[0].view_3d.object_active = self.user_active_color
                bpy.context.space_data.shading.color_type = self.user_color_type
                bpy.context.space_data.shading.type = self.user_shading_type
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')

                return {'FINISHED'}

            elif event.type in {'RIGHTMOUSE', 'ESC'}:
                bpy.context.object.data.uv_layers["UVMap"].active = True
                bpy.context.object.data.uv_layers["UVMap"].active_render = True
                context.preferences.themes[0].view_3d.object_active = self.user_active_color
                bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
                context.space_data.shading.color_type = self.user_color_type
                context.space_data.shading.type = self.user_shading_type
                return {'CANCELLED'}

        except Exception as e:
            obj = bpy.context.object
            if obj.mode == "EDIT":
                bpy.ops.object.editmode_toggle()
            bpy.context.object.data.uv_layers["UVMap"].active = True
            bpy.context.object.data.uv_layers["UVMap"].active_render = True
            context.preferences.themes[0].view_3d.object_active = self.user_active_color
            bpy.types.SpaceView3D.draw_handler_remove(self._handle, 'WINDOW')
            context.space_data.shading.color_type = self.user_color_type
            context.space_data.shading.type = self.user_shading_type
            raise e

        return {'RUNNING_MODAL'}
    def invoke(self, context, event):

        self.user_active_color = context.preferences.themes[0].view_3d.object_active
        self.user_color_type  = context.space_data.shading.color_type
        self.user_shading_type = context.space_data.shading.type

        if context.object:
            bpy.context.object.data.uv_layers["UVMap_Displace"].active = True
            bpy.context.object.data.uv_layers["UVMap_Displace"].active_render = True

            Color5 = preferences.get_preferences().color_Modal_Displace[:-1]
            bpy.context.preferences.themes[0].view_3d.object_active = Color5

            bpy.context.space_data.shading.color_type = 'MATERIAL'
            bpy.context.space_data.shading.type = 'SOLID'

            self.first_value = bpy.context.object.modifiers["Displace_map"].strength
            self.first_value_blur = bpy.context.object.modifiers["Displace_map"].texture.filter_size

            args = (self, context)
            self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_displace, args, 'WINDOW', 'POST_PIXEL')
            # self._handle = bpy.types.SpaceView3D.draw_handler_add(draw_modal_displace_shortcuts, args, 'WINDOW', 'POST_PIXEL')
            context.window_manager.modal_handler_add(self)
            return {'RUNNING_MODAL'}

class QUP_OT_copy_gizmo_modifier(bpy.types.Operator):
    """Copy Gizmo from Active to Selected"""
    bl_idname = "qup.copy_gizmo_modifier"
    bl_label = "Copy Gizmo to Selected"

    param_gizmo: StringProperty()
    def execute(self, context,):

        if self.param_gizmo == 'C':
            bpy.ops.object.modifier_copy_to_selected(modifier="QuickUVProject_Cube")
        if self.param_gizmo == 'P':
            bpy.ops.object.modifier_copy_to_selected(modifier="QuickUVProject_Plane")

        return {'FINISHED'}

gizmo_used = []

class QUP_OT_world_gizmo_switch(bpy.types.Operator):
    '''Shortcut_popup'''
    bl_label = "Gz_popup"
    bl_idname = "qup.world_gizmo_switch"

    def invoke(self, context, event):
        obj = bpy.context.object
        obj_parent = obj.parent

        used_mats = set(obj.material_slots[f.material_index].material
                        for f in obj.data.polygons)

        gizmos = bpy.data.collections["UV_GIZMO"].all_objects
        Gz = [giz for giz in gizmos if giz in obj.parent.children and giz.name.startswith('Gizmo_WORLD') and used_mats]

        i = Gz.index(obj)
        if event.type == 'UP_ARROW' or 'A':
            for giz in Gz:
                if giz != obj:
                    giz.select_set(False)
                giz.hide_viewport = True

                if i + 1 < len(Gz):
                    bpy.context.view_layer.objects.active = Gz[i + 1]
                    Gz[i + 1].hide_viewport = False
                    Gz[i + 1].select_set(True)

                else:
                    bpy.context.view_layer.objects.active = Gz[0]
                    Gz[0].select_set(True)
                    Gz[0].hide_viewport = False

        if event.type == 'DOWN_ARROW':
            for giz in Gz:
                if giz != obj:
                    giz.select_set(False)
                giz.hide_viewport = True

                if not i < 0:
                    bpy.context.view_layer.objects.active = Gz[i-1]
                    Gz[i -1].hide_viewport = False
                    Gz[i -1].select_set(True)

                else:
                    bpy.context.view_layer.objects.active = Gz.pop()
                    Gz[0].select_set(True)
                    Gz[0].hide_viewport = False


        return {'FINISHED'}



class Qup_prop(bpy.types.PropertyGroup,):
    user_active_color: FloatVectorProperty()
    user_wire_color: FloatVectorProperty()
    user_active_object: StringProperty
    user_loc: FloatVectorProperty()
    user_rot: FloatVectorProperty()
    user_sca: FloatVectorProperty()
    user_proportional: BoolProperty()


classes = {
    Qup_prop,
    QUP_OT_modal_uv,
    QUP_OT_uv_cube_proj,
    QUP_OT_uv_from_view,
    QUP_OT_uv_cylinder_proj,
    QUP_OT_uv_sphere_proj,
    QUP_OT_apply_mesh,
    QUP_OT_apply_gizmo,
    QUP_OT_uv_straight_unwrap,
    QUP_OT_copy_material,
    QUP_OT_add_gizmo_world,
    QUP_OT_gizmo_cube,
    QUP_OT_gizmo_plane,
    QUP_OT_align_view_to_face,
    QUP_OT_rotate_uv,
    QUP_OT_rotate_uv_deg,
    QUP_OT_handle_generated,
    QUP_OT_swap_textures,
    QUP_OT_change_img_texture,
    QUP_OT_draw_qup_mat,
    QUP_OT_one_texture_group,
    QUP_OT_world_gizmo_switch,

    QUP_OT_gizmo_lock_ratio,
    QUP_OT_gizmo_dimensions,
    QUP_OT_modal_add_mat_nw,
    QUP_OT_add_one_texture_material,
    QUP_OT_add_world_material,
    QUP_OT_select_world_gizmo,
    QUP_OT_select_cube_gizmo,
    QUP_OT_select_plane_gizmo,
    QUP_OT_fit_gizmo_to_mesh,
    QUP_OT_fit_gizmo_selected_faces,
    QUP_OT_displace_map,
    QUP_OT_displace_modal,
    QUP_OT_copy_gizmo_modifier,
    QUP_OT_warning_straight_unwrap,


}

addon_keymaps = []

def register():

    auto_reaload.init()
    auto_reaload.reload()
    custom_icons.register()
    quick_image.register()
    preferences.register()
    keymaps.register()
    qup_pie_menus.register()
    light_texture.register()
    light_preview.register()
    terrain.register()

    for cls in classes:
        bpy.utils.register_class(cls)

    # bpy.types.Scene.qup = bpy.props.PointerProperty(type=Qup_prop)

def unregister():
    custom_icons.unregister()
    quick_image.unregister()
    preferences.unregister()
    keymaps.unregister()
    qup_pie_menus.unregister()
    light_texture.unregister()
    light_preview.unregister()
    terrain.unregister()

    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()

    for cls in classes:
        bpy.utils.unregister_class(cls)

    # del bpy.types.Scene.qup

    auto_reaload.reload()

