import bpy

addon_keymaps = []


def add_hotkey(keymap, space, region, identifier, event_type, value, property=None, ctrl=False, shift=False,
               alt=False, ):
    if bpy.context.window_manager:
        wm = bpy.context.window_manager
        if wm.keyconfigs.addon:
            kc = wm.keyconfigs.addon
            km = kc.keymaps.new(name=keymap, space_type=space, region_type=region)
            kmi = km.keymap_items.new(identifier, event_type, value,
                                      ctrl=ctrl, shift=shift, alt=alt,
                                      )
            if property:
                kmi.properties.name = property
            kmi.active = True
            addon_keymaps.append((km, kmi))


def remove_hotkey(keymap):
    ''' clears all addon level keymap hotkeys stored in addon_keymaps '''
    if bpy.context.window_manager:
        wm = bpy.context.window_manager
        if wm.keyconfigs.addon:
            kc = wm.keyconfigs.addon
            if kc.keymaps.get(keymap, False):

                km = kc.keymaps[keymap]

                for km, kmi in addon_keymaps:
                    km.keymap_items.remove(kmi)
                addon_keymaps.clear()


def get_hotkey_entry_item(km, kmi_name, kmi_value):
    """
    returns hotkey of specific type, with specific properties.name (keymap is not a dict, so referencing by keys is not enough
    if there are multiple hotkeys!)
    """
    for i, km_item in enumerate(km.keymap_items):
        if km.keymap_items.keys()[i] == kmi_name:
            if kmi_value:
                if km.keymap_items[i].properties.name == kmi_value:
                    return km_item
                else:
                    return None
            else:
                return km_item
    return None


def register():
    add_hotkey("3D View", 'VIEW_3D', 'WINDOW',
               "wm.call_menu_pie",
               'B', 'PRESS', property="QUP_MT_pie_uv_pro", )

    add_hotkey("3D View", 'VIEW_3D', 'WINDOW',
               "qup.modal_uv",
               'MIDDLEMOUSE', 'PRESS', ctrl=True)

    add_hotkey("3D View", 'VIEW_3D', 'WINDOW',
               "qup.gizmo_dimensions",
               'RIGHTMOUSE', 'PRESS', )

    add_hotkey("3D View", 'VIEW_3D', 'WINDOW',
               'Quick Uv Pro: Validate Modal',
               'BUTTON4MOUSE', 'PRESS')

    add_hotkey("3D View", 'VIEW_3D', 'WINDOW',
               'qup.draw_qup_mat',
               'M', 'PRESS', alt=True)

    add_hotkey("3D View", 'VIEW_3D', 'WINDOW',
               'qup.terrain',
               'T', 'PRESS', alt=True)

    # add_hotkey("Node Editor", 'NODE_EDITOR', 'WINDOW',
    #            "wm.call_menu_pie",
    #            'BUTTON4MOUSE', 'PRESS', property="QUP_MT_pie_images_nodes", ctrl=True)

    add_hotkey("Node Editor", 'NODE_EDITOR', 'WINDOW',
               "wm.call_menu_pie",
               'B', 'PRESS', property="QUP_MT_pie_images_nodes", )



    wm = bpy.context.window_manager
    km = wm.keyconfigs.addon.keymaps.new(name="Mesh")
    kmi = km.keymap_items.new("wm.call_menu_pie", type="B", value="PRESS")
    kmi.properties.name = "QUP_MT_pie_uv_pro"

    addon_keymaps.append((km, kmi))


def unregister():
    remove_hotkey("3D View")
    remove_hotkey("Node Editor")
