import bpy
import os
from bpy.props import(StringProperty, BoolProperty, EnumProperty)
from bpy.props import *
import bpy.utils.previews
from bpy.types import WindowManager

preview_collections = {}
def enum_previews_light_texture_from_directory_items(self, context):
    # Extensions
    extensions = ('.jpeg', '.jpg', '.png', '.psd', '.tga')

    directory = context.preferences.addons[__package__].preferences.light_texture_path
    enum_items = []

    if context is None:
        return enum_items

    pcoll = preview_collections["light"]

    if directory == pcoll.my_previews_dir:
        return pcoll.my_previews

    if directory and os.path.exists(directory):
        # Scan the directory for png files
        image_paths = []
        for fn in os.listdir(directory):
            if fn.lower().endswith(extensions):
                image_paths.append(fn)

        for i, name in enumerate(image_paths):
            # generates a thumbnail preview for a file.
            filepath = os.path.join(directory, name)
            icon = pcoll.get(name)
            if filepath in pcoll:
                enum_items.append((name, name, "", pcoll[filepath].icon_id, i))
            else:
                thumb = pcoll.load(filepath, filepath, 'IMAGE')
                enum_items.append((name, name, "", thumb.icon_id, i))

    pcoll.my_previews = enum_items
    pcoll.my_previews_dir = directory

    return pcoll.my_previews

def enum_previews_ies_texture_from_directory_items(self, context):
    # Extensions
    extensions = ('.jpeg', '.jpg', '.png', '.psd', '.tga')

    directory = context.preferences.addons[__package__].preferences.ies_texture_path
    enum_items = []

    if context is None:
        return enum_items

    pcoll2 = preview_collections["ies"]

    if directory == pcoll2.my_previews_dir2:
        return pcoll2.my_previews2

    if directory and os.path.exists(directory):
        # Scan the directory for png files
        image_paths = []
        for fn in os.listdir(directory):
            if fn.lower().endswith(extensions):
                image_paths.append(fn)

        for i, name in enumerate(image_paths):
            # generates a thumbnail preview for a file.
            filepath = os.path.join(directory, name)
            icon = pcoll2.get(name)
            if filepath in pcoll2:
                enum_items.append((name, name, "", pcoll2[filepath].icon_id, i))
            else:
                thumb = pcoll2.load(filepath, filepath, 'IMAGE')
                enum_items.append((name, name, "", thumb.icon_id, i))

    pcoll2.my_previews2 = enum_items
    pcoll2.my_previews_dir2 = directory

    return pcoll2.my_previews2

def preview_light_texture_enum_update(self, context):
    obj = bpy.context.object
    bpy.context.object.data.use_nodes = True

    light = bpy.context.active_object.data
    nodes = light.node_tree.nodes
    tree = light.node_tree

    pref_light_path = context.preferences.addons[__package__].preferences.light_texture_path
    found = False
    for node in nodes:
        if node.type == 'TEX_IMAGE':
            found = True
    if not found:

        Texture_Node = nodes.new('ShaderNodeTexImage')
        mapping = nodes.new('ShaderNodeMapping')
        Coord = nodes.new('ShaderNodeTexCoord')
        Emission_Node = nodes.get('Emission')

        tree.links.new(Coord.outputs[1], mapping.inputs[0])
        tree.links.new(mapping.outputs[0], Texture_Node.inputs[0])
        tree.links.new(Texture_Node.outputs[0], Emission_Node.inputs[0])
        mapping.vector_type = 'TEXTURE'

        Emission_Node.inputs[1].default_value = 50
        mapping.inputs[1].default_value[0] = .5
        mapping.inputs[1].default_value[1] = .5

        Texture_Node.location = (-300, 215)
        mapping.location = (-500, 215)
        Coord.location = (-900, 215)

        Texture_Node.select = True
        nodes.active = Texture_Node

        selected_preview = obj.qup.previews_light_texture

        base_path = os.path.join(pref_light_path, selected_preview)
        i = bpy.data.images.load(base_path)
        Texture_Node.image = i

    else:
        for node in nodes:
            if node.type == 'TEX_IMAGE':
                node = node
                selected_preview = obj.qup.previews_light_texture
                base_path = os.path.join(pref_light_path, selected_preview)
                i = bpy.data.images.load(base_path)
                node.image = i


    return None

def preview_ies_texture_enum_update(self, context):
    obj = bpy.context.object
    bpy.context.object.data.use_nodes = True

    light = bpy.context.active_object.data
    nodes = light.node_tree.nodes
    tree = light.node_tree

    pref_light_path = context.preferences.addons[__package__].preferences.ies_texture_path
    found = False
    for node in nodes:
        if node.type == 'TEX_IES':
            found = True
    if not found:
        bpy.context.object.data.type = 'POINT'
        bpy.context.object.data.shadow_soft_size = 0

        ies = nodes.new('ShaderNodeTexIES')
        ies.mode = 'EXTERNAL'
        Emission_Node = nodes.get('Emission')

        tree.links.new(ies.outputs[0], Emission_Node.inputs[1])

        Emission_Node.inputs[1].default_value = 50

        ies.location = (-300, 215)
        ies.inputs[1].default_value = .01
        nodes.active = ies

        selected_preview = obj.qup.previews_ies_texture
        preview_no_ext = selected_preview.split('.')[0]
        ies_name = preview_no_ext + '.ies'
        base_path = os.path.join(pref_light_path, ies_name)
        ies.filepath = base_path
    else:
        for node in nodes:
            if node.type == 'TEX_IES':
                node=node
                selected_preview = obj.qup.previews_ies_texture
                preview_no_ext = selected_preview.split('.')[0]

                ies_name = preview_no_ext + '.ies'
                base_path = os.path.join(pref_light_path, ies_name)
                node.filepath = base_path

    return None


class Qup_light_texture_preview(bpy.types.PropertyGroup):

    # WindowManager.my_previews_img_dir = StringProperty(
    previews_img_dir: StringProperty(
        name="Folder Path",
        subtype='DIR_PATH',
        default=""
    )

    previews_light_texture: EnumProperty(
        items=enum_previews_light_texture_from_directory_items,
        default=0,
        update=preview_light_texture_enum_update
    )

    previews_ies_texture: EnumProperty(
        items=enum_previews_ies_texture_from_directory_items,
        default=0,
        update=preview_ies_texture_enum_update
    )

    pcoll = bpy.utils.previews.new()
    pcoll.my_previews_dir = ""
    pcoll.my_previews = ()

    pcoll2 = bpy.utils.previews.new()
    pcoll2.my_previews_dir2 = ""
    pcoll2.my_previews2 = ()

    preview_collections["light"] = pcoll
    preview_collections["ies"] = pcoll2


classes = [
            Qup_light_texture_preview,
           ]

def register():

    for cls in classes:
        bpy.utils.register_class(cls)
        # bpy.types.WindowManager.qup = bpy.props.PointerProperty(type=Qup_light_texture_preview)
        bpy.types.Object.qup = bpy.props.PointerProperty(type=Qup_light_texture_preview)

def unregister():
    global index_prev
    for cls in classes:
        bpy.utils.unregister_class(cls)

    # del bpy.types.WindowManager.qup
    del bpy.types.Object.qup

    # for pcoll in preview_collections.values():
    #     bpy.utils.previews.remove(pcoll)

    preview_collections.clear()
