import bpy
import os
from bpy_extras.io_utils import ImportHelper
from bpy.props import StringProperty

class QUP_OT_light_texture(bpy.types.Operator, ImportHelper):
    '''Add Projected Texture to Light'''
    bl_label = "Light Texture"
    bl_idname = "qup.light_texture"

    directory: StringProperty(subtype='DIR_PATH')

    def execute(self, context):
        obj = bpy.context.object
        bpy.context.object.data.use_nodes = True

        light = bpy.context.active_object.data
        nodes = light.node_tree.nodes
        tree = light.node_tree

        found = False
        for node in nodes:
            if node.type == 'TEX_IMAGE':
                found = True
        if not found:
            obj.data.use_nodes = True

            Texture_Node = nodes.new('ShaderNodeTexImage')
            mapping = nodes.new('ShaderNodeMapping')
            Coord = nodes.new('ShaderNodeTexCoord')
            Emission_Node = nodes.get('Emission')

            tree.links.new(Coord.outputs[1], mapping.inputs[0])
            tree.links.new(mapping.outputs[0], Texture_Node.inputs[0])
            tree.links.new(Texture_Node.outputs[0], Emission_Node.inputs[0])

            mapping.vector_type = 'TEXTURE'

            Emission_Node.inputs[1].default_value = 50
            mapping.inputs[1].default_value[0] = .5
            mapping.inputs[1].default_value[1] = .5

            Texture_Node.location = (-300, 215)
            mapping.location = (-500, 215)
            Coord.location = (-900, 215)

            Texture_Node.select = True
            nodes.active = Texture_Node
            img = bpy.data.images.load(self.filepath)
            Texture_Node.image = img

        else:
            img = bpy.data.images.load(self.filepath)
            node.image = img


        return {'FINISHED'}

class QUP_OT_ies_light_texture(bpy.types.Operator, ImportHelper):
    '''Add Projected Texture to Light'''
    bl_label = "IES Light Texture"
    bl_idname = "qup.ies_light_texture"

    directory: StringProperty(subtype='DIR_PATH')

    def execute(self, context):

        bpy.context.object.data.use_nodes = True

        light = bpy.context.active_object.data
        nodes = light.node_tree.nodes
        tree = light.node_tree

        bpy.context.object.data.type = 'POINT'

        found = False
        for node in nodes:
            if node.type == 'TEX_IES':
                found = True
        if not found:
            ies = nodes.new('ShaderNodeTexIES')

            ies.mode = 'EXTERNAL'
            ies.filepath = self.filepath
            Emission_Node = nodes.get('Emission')

            tree.links.new(ies.outputs[0], Emission_Node.inputs[1])

            Emission_Node.inputs[1].default_value= 50

            ies.location = (-300, 215)
            ies.inputs[1].default_value = .01
            nodes.active = ies
        else:
            node.filepath = self.filepath


        return {'FINISHED'}

class QUP_OT_change_light_texture(bpy.types.Operator, ImportHelper):
    '''Change Projected Texture Light'''
    bl_label = "Change Light Texture"
    bl_idname = "qup.change_light_texture"

    def execute(self, context):

        light = bpy.context.active_object.data
        nodes = light.node_tree.nodes

        n = nodes["Image Texture"]

        nodes.active = n
        img = bpy.data.images.load(self.filepath)
        n.image = img

        return {'FINISHED'}


register, unregister = bpy.utils.register_classes_factory((
        QUP_OT_light_texture,
        QUP_OT_ies_light_texture,
        QUP_OT_change_light_texture
        ))
