import bpy
import rna_keymap_ui

from bpy.types import AddonPreferences
from bpy.props import (
    BoolProperty,
    IntProperty,
    EnumProperty,
    FloatProperty,
    FloatVectorProperty,
    StringProperty,
)
from .keymaps import get_hotkey_entry_item

class QuickUvPro_Preferences(AddonPreferences):
    bl_idname = __package__

    tab: EnumProperty(
        name="Preferences Tab",
        items=[('UI', 'UI', ''),
               ('KEYMAPS', 'Keymaps', ''),
               ('PATHS', 'Paths', ''),
               ('DEMOS', 'Demos', ''),

               ],
        default='UI',
    )

    Display_Shortcuts_Modal_Uv: BoolProperty(

        default=True,
    )

    x_position_shortcuts: IntProperty(
        # default=190,
        default=130,
    )

    y_position_shortcuts: IntProperty(
        # default=430,
        default=300,
    )

    size_shortcuts: IntProperty(
        default=17,
    )

    x_modal_shortcuts: IntProperty(
        # default=1015,
        default=650,
    )

    y_modal_shortcuts: IntProperty(
        # default=150,
        default=120,
    )

    size_modal_shortcuts: IntProperty(
        # default=35,
        default=25,
    )

    Display_Experimental: BoolProperty(

        default=False,
    )

    Text_Color_Modal_Uv: FloatVectorProperty(
        default=(0, .8, 1, 1),
        min=0,
        max=1,
        size=4,
        subtype='COLOR_GAMMA'
    )

    Color_World_Gizmo: FloatVectorProperty(
        default=(1, .6, .1, 1),
        min=0,
        max=1,
        size=4,
        subtype='COLOR_GAMMA'
    )

    Face_opacity_Modal_Uv: FloatProperty(
        default=0.06,
        min=0.03,
        max=.1,

    )

    Custom_Color_Gizmo: FloatVectorProperty(
        description="!!! Change All Empties Color",
        default=(0, 0, 0, 1),
        min=0,
        max=1,
        size=4,
        subtype='COLOR_GAMMA'
    )
    color_Modal_Displace: FloatVectorProperty(
        default=(.8, .5, 1, 1),
        min=0,
        max=1,
        size=4,
        subtype='COLOR_GAMMA')

    light_texture_path: StringProperty(
        name="Light Texture Path",
        description="Path to Light Textures",
        subtype='DIR_PATH',
    )
    ies_texture_path: StringProperty(
        name="Light Texture Path",
        description="Path to ies Textures",
        subtype='DIR_PATH',
    )

    def draw(self, context):

        layout = self.layout
        row = layout.row(align=True)
        row.prop(self, "tab", expand=True)
        pref = bpy.context.preferences.addons[__package__].preferences

        if self.tab == "UI":
            paths = context.preferences.filepaths
            box = layout.box()
            row = box.row(align=True)
            row.label(text='Interface Settings:', )



            row = box.row(align=True)
            row.prop(self, "Text_Color_Modal_Uv", text='Color for Modal Uv and Uv Gizmo')
            row = box.row(align=True)
            row.prop(self, "Color_World_Gizmo", text='Color for World Gizmo')
            row = box.row(align=True)
            row.label(text='Selected Faces opacity for Modal Uv:', )
            row.prop(self, "Face_opacity_Modal_Uv", text='')
            if pref.Display_Experimental:
                row = box.row(align=True)
                row.label(text='Custom Color of Modal Displace:', )
                row.prop(self, "color_Modal_Displace", text='')

            box = layout.box()
            row = box.row(align=True)
            row.label(text='Shortcuts Positions and Sizes')
            row = box.row(align=True)
            row.prop(self, "Display_Shortcuts_Modal_Uv", text='Dispaly Shortcuts for Modal Uv and Modal Gizmos')
            row = box.row(align=True)
            row.label(text='Shortcuts:')
            row.prop(self, "x_position_shortcuts", text='X')
            row.prop(self, "y_position_shortcuts", text='Y')
            row.prop(self, "size_shortcuts", text='Size')
            row = box.row(align=True)
            row.label(text='Title:')
            row.prop(self, "x_modal_shortcuts", text='X')
            row.prop(self, "y_modal_shortcuts", text='Y')
            row.prop(self, "size_modal_shortcuts", text='Size')

            # box = layout.box()
            # row = box.row(align=True)
            # row.prop(self, "Display_Experimental", text='Experimental Tools')


        if self.tab == "KEYMAPS":

            box = layout.box()
            box.label(text="Quick Uv Pro Hotkeys:")
            wm = bpy.context.window_manager

            kc = wm.keyconfigs.user
            km = kc.keymaps['3D View']

            kmi = get_hotkey_entry_item(km, "wm.call_menu_pie", "QUP_MT_pie_uv_pro", )
            if kmi:
                box.context_pointer_set("keymap", km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, box, 0)

            kmi = get_hotkey_entry_item(km, "qup.modal_uv", None)
            if kmi:
                box.context_pointer_set("keymap", km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, box, 0)

            kmi = get_hotkey_entry_item(km, "qup.gizmo_dimensions", None)
            if kmi:
                box.context_pointer_set("keymap", km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, box, 0)

            box.separator()

            kmi = get_hotkey_entry_item(km, "Quick Uv Pro: Validate Modal", None)
            if kmi:
                box.context_pointer_set("keymap", km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, box, 0)

            box.separator()

            kmi = get_hotkey_entry_item(km, "qup.draw_qup_mat", None)
            if kmi:
                box.context_pointer_set("keymap", km)
                rna_keymap_ui.draw_kmi([], kc, km, kmi, box, 0)

            # box.separator()
            #
            # if get_preferences().Display_Experimental:
            #     kmi = get_hotkey_entry_item(km, "qup.terrain", None)
            #     if kmi:
            #         box.context_pointer_set("keymap", km)
            #         rna_keymap_ui.draw_kmi([], kc, km, kmi, box, 0)

            kc = wm.keyconfigs.user
            km2 = kc.keymaps['Node Editor']
            kmi = get_hotkey_entry_item(km2, 'wm.call_menu_pie', 'QUP_MT_pie_images_nodes')
            if kmi:
                box.context_pointer_set("keymap", km2)
                rna_keymap_ui.draw_kmi([], kc, km2, kmi, box, 0)



        if self.tab == "PATHS":
            paths = context.preferences.filepaths
            box = layout.box()
            box.label(text="Set your Custom Paths:")
            row = box.row(align=True)
            row.prop(paths, "image_editor", text="Image Editor")
            row = box.row(align=True)
            row.prop(self, "light_texture_path", text="light_texture_path")
            row = box.row(align=True)
            row.prop(self, "ies_texture_path", text="ies_texture_path")

        if self.tab == "DEMOS":
            box = layout.box()
            box.label(text="Check the demos here:")
            box.operator("wm.url_open", text="Quick Uv Pro 3.0 /// YOU TUBE DEMO").url ="https://www.youtube.com/watch?v=5OBNUBV6UMI"
            box.label(text="Other Addons:")
            box.operator("wm.url_open",
                         text="Add On The Fly").url = "https://www.youtube.com/watch?v=dELco-8itdk"


def get_preferences():
    return bpy.context.preferences.addons[__package__].preferences


register, unregister = bpy.utils.register_classes_factory((QuickUvPro_Preferences,))