import bpy
import os
from os import listdir
from bpy.props import (StringProperty, BoolProperty, EnumProperty, IntProperty, FloatProperty)
from bpy.props import *
import bpy.utils.previews
from.utils import nodes_utils

# from .qup_pie_menus import global index_prev
from bpy.types import WindowManager

# preview_collections = {}
# index_prev = [0]
tex_nodes=['TEX_IMAGE',
          'TEX_BRICK',
          'TEX_CHECKER',
          'TEX_IES',
          'TEX_MAGIC',
          'TEX_MUSGRAVE',
          'TEX_NOISE',
          'TEX_POINTDENSITY',
          'TEX_VORONOI',
          'TEX_WAVE',
          'TEX_WHITE_NOISE']

class QUP_OT_launch_external_soft(bpy.types.Operator):
    '''Export and Create Thumbnails'''
    bl_label = "Studio Render"
    bl_idname = "qup.launch_external_soft"

    def execute(self, context):
        i_shader = context.active_node
        print(i_shader.name)

        img = i_shader.image

        full_path = bpy.path.abspath(img.filepath, library=img.library)

        bpy.ops.image.external_edit(filepath=full_path)

        return {'FINISHED'}


class QUP_OT_reload_img(bpy.types.Operator):
    '''Export and Create Thumbnails'''
    bl_label = "Studio Render"
    bl_idname = "qup.reload_img"

    def execute(self, context):
        bpy.ops.image.reload()

        return {'FINISHED'}

class QUP_OT_pack_img(bpy.types.Operator):
    '''Export and Create Thumbnails'''
    bl_label = "Pack Image into Blend File"
    bl_idname = "qup.pack_img"

    def execute(self, context):
        i_shader = context.object.active_material.node_tree.nodes.active
        i_shader.image.pack()

        return {'FINISHED'}

class QUP_OT_unpack_img(bpy.types.Operator):
    '''Export and Create Thumbnails'''
    bl_label = "Unpack Image into Blend File"
    bl_idname = "qup.unpack_img"

    def execute(self, context):
        i_shader = context.object.active_material.node_tree.nodes.active
        i_shader.image.unpack()

        return {'FINISHED'}

class QUP_OT_relative_img(bpy.types.Operator):
    '''Export and Create Thumbnails'''
    bl_label = "Import Image In The Relative Texture folder"
    bl_idname = "qup.relative_img"

    def execute(self, context):
        i_shader = context.object.active_material.node_tree.nodes.active
        i_shader.image.pack()
        i_shader.image.unpack()

        return {'FINISHED'}

class QUP_OT_add_displace_node(bpy.types.Operator):
    '''Add Displace Node'''
    bl_label = "Add Displace Node"
    bl_idname = "qup.add_displace_node"

    def execute(self, context):
        mat= bpy.context.object.active_material
        n = bpy.context.object.active_material.node_tree.nodes.active
        Output_Node = mat.node_tree.nodes.get('Material Output')
        # if n.type == 'TEX_IMAGE':
        if n.type in tex_nodes:
            disp_node = mat.node_tree.nodes.new(type="ShaderNodeDisplacement")
            disp_node.location[0] = Output_Node.location[0]-250
            disp_node.location[1] = Output_Node.location[1]-250
            n_link = mat.node_tree.links.new

            n_link(n.outputs[0], disp_node.inputs[0])
            n_link(disp_node.outputs[0], Output_Node.inputs['Displacement'])

            disp_node.inputs[2].default_value = .1

        # i_shader = context.object.active_material.node_tree.nodes.active
        # i_shader.image.pack()
        # i_shader.image.unpack()

        return {'FINISHED'}

class QUP_OT_add_bump_node(bpy.types.Operator):
    '''Add Bump Node'''
    bl_label = "Add Displace Node"
    bl_idname = "qup.add_bump_node"

    def execute(self, context):
        mat = bpy.context.object.active_material
        n = bpy.context.object.active_material.node_tree.nodes.active
        bsdf_Node = mat.node_tree.nodes.get('Principled BSDF')

        if n.type in tex_nodes:
            bmp_node = mat.node_tree.nodes.new(type="ShaderNodeBump")

            if bsdf_Node.location[0]-n.location[0]<550:
                n.location[0]= n.location[0]-350

            bmp_node.location[0] = n.location[0] + 450
            bmp_node.location[1] = n.location[1] -250

            n_link = mat.node_tree.links.new
            n_link(n.outputs[0], bmp_node.inputs[2])
            n_link(bmp_node.outputs[0], bsdf_Node .inputs[22])

            bmp_node.inputs["Strength"].default_value=.5
            if n.type == 'TEX_IMAGE':
                n.interpolation = 'Smart'

        return {'FINISHED'}


class QUP_OT_add_normal_node(bpy.types.Operator):
    '''Add NormalMap Nodes'''
    bl_label = "Add Normal Map Nodes"
    bl_idname = "qup.add_normal_node"

    def execute(self, context):
        mat = bpy.context.object.active_material
        n = bpy.context.object.active_material.node_tree.nodes.active
        bsdf_Node = mat.node_tree.nodes.get('Principled BSDF')

        if n.type in tex_nodes:
            nrm_node = mat.node_tree.nodes.new(type="ShaderNodeNormalMap")

            if bsdf_Node.location[0] - n.location[0] < 550:
                n.location[0] = n.location[0] - 350

            nrm_node.location[0] = n.location[0] + 450
            nrm_node.location[1] = n.location[1] - 250

            n_link = mat.node_tree.links.new
            n_link(n.outputs[0], nrm_node.inputs[1])
            n_link(nrm_node.outputs[0], bsdf_Node.inputs[22])

            nrm_node.inputs["Strength"].default_value = .5
            n.image.colorspace_settings.name = 'Non-Color'

        return {'FINISHED'}

class QUP_OT_add_occlusion_node(bpy.types.Operator):
    '''Add NormalMap Nodes'''
    bl_label = "Add Occlusion Nodes"
    bl_idname = "qup.add_occlusion_node"

    def execute(self, context):
        mat = bpy.context.object.active_material
        n = bpy.context.object.active_material.node_tree.nodes.active
        bsdf_node = mat.node_tree.nodes.get('Principled BSDF')
        One_Shader_Node = mat.node_tree.nodes.get('Group')

        if bsdf_node is None:
            bsdf_node = One_Shader_Node


        if n.type in tex_nodes:

            grd_node = mat.node_tree.nodes.new(type="ShaderNodeValToRGB")
            mix_node = mat.node_tree.nodes.new(type="ShaderNodeMix")


            if bsdf_node.location[0] - n.location[0] < 950:
                n.location[0] = bsdf_node.location[0] - 950


            grd_node.location[0] = n.location[0] + 350
            grd_node.location[1] = n.location[1] + 250
            mix_node.location[0] = n.location[0] + 650
            mix_node.location[1] = n.location[1] + 250

            n_link = mat.node_tree.links.new
            n_link(n.outputs[0], grd_node.inputs[0])
            n_link(n.outputs[0], mix_node.inputs[1])
            n_link(grd_node.outputs[0], mix_node.inputs[2])
            n_link(mix_node.outputs[0], bsdf_node.inputs[0])

            mix_node.blend_type = 'MULTIPLY'
            # bpy.ops.node.select_all(action='TOGGLE')
            frame = mat.node_tree.nodes.new(type='NodeFrame')
            frame.label ='Map Occlusion'
            grd_node.parent = frame
            mix_node.parent = frame


        return {'FINISHED'}

class QUP_OT_replace_onetexturenode(bpy.types.Operator):
    '''Replace One Texture Node by Principled BSDF'''
    bl_label = "Replace One Texture Node"
    bl_idname = "qup.replace_onetexturenode"

    def execute(self, context):
        mat = bpy.context.object.active_material
        nt = context.object.active_material.node_tree
        nodes = nt.nodes
        bpy.ops.node.nw_swtch_node_type(to_type='ShaderNodeBsdfPrincipled')
        sel = [x for x in nodes if x.select]
        bsdf_node = sel[0]
        i = bsdf_node.inputs[0].links[0].from_node
        context.object.active_material.node_tree.nodes.active=i
        bpy.ops.qup.add_displace_node()
        context.object.active_material.node_tree.nodes.active=bsdf_node

        return {'FINISHED'}
class QUP_OT_replace_p_bsdf(bpy.types.Operator):
    '''Replace Principled BSDF by a One Texture Node  '''
    bl_label = "Replace Principled BSDF by One Texture Node"
    bl_idname = "qup.replace_p_bsdf"

    posx_bsdf: FloatProperty()
    posy_bsdf: FloatProperty()
    def execute(self, context):
        mat = bpy.context.object.active_material
        n = mat.node_tree.nodes.active
        i = n.inputs[0].links[0].from_node
        Output_Node = mat.node_tree.nodes.get('Material Output')
        self.posx_bsdf = n.location[0]
        self.posy_bsdf = n.location[1]

        bpy.ops.node.delete()
        if bpy.context.object.mode == "EDIT":
            bpy.ops.object.editmode_toggle()
        # if bpy.context.object.mode != "EDIT":
        # if n.type == 'TEX_IMAGE':

        file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")
        inner_path = 'NodeTree'
        nodetree_name = 'One_Texture_Shader'

        bpy.ops.wm.append(
            filepath=os.path.join(file_path, inner_path, nodetree_name),
            directory=os.path.join(file_path, inner_path),
            filename=nodetree_name
        )
        One_Shader_Node = mat.node_tree.nodes.new(type='ShaderNodeGroup')
        One_Shader_Node.node_tree = bpy.data.node_groups['One_Texture_Shader']

        One_Shader_Node.location[0] = self.posx_bsdf
        One_Shader_Node.location[1] = self.posy_bsdf
        n_link = mat.node_tree.links.new
        #
        # n_link(n.outputs[0], One_Shader_Node.inputs[0])
        n_link(i.outputs[0], One_Shader_Node.inputs[0])
        n_link(One_Shader_Node.outputs[0], Output_Node.inputs['Surface'])
        n_link(One_Shader_Node.outputs[1], Output_Node.inputs['Displacement'])

        return {'FINISHED'}

class QUP_OT_add_bevel_node(bpy.types.Operator):
    '''Add Bevel Node '''
    bl_label = "Add Bevel Node"
    bl_idname = "qup.add_bevel_node"

    def execute(self, context):
        mat = bpy.context.object.active_material
        nt = context.object.active_material.node_tree
        nodes = nt.nodes
        node=nodes.active

        bsdf_Node = mat.node_tree.nodes.get('Principled BSDF')
        if bsdf_Node:

            bv_node = mat.node_tree.nodes.new(type="ShaderNodeBevel")
            bv_node.inputs[0].default_value = .003
            bv_node.samples = 15

            bv_node.location[0] = bsdf_Node.location[0] - 250
            bv_node.location[1] = bsdf_Node.location[1] - 600

            n_link = mat.node_tree.links.new
            n_link(bv_node.outputs[0], bsdf_Node.inputs[22])

        # sel = [x for x in nodes if x.select]
        # bsdf_node = sel[0]
        # i = bsdf_node.inputs[0].links[0].from_node
        # context.object.active_material.node_tree.nodes.active=i
        # bpy.ops.qup.add_displace_node()
        # context.object.active_material.node_tree.nodes.active=bsdf_node
        return {'FINISHED'}
class QUP_OT_add_simple_ao(bpy.types.Operator):
    '''Add Simple AO Node '''
    bl_label = "Add Simple AO Node"
    bl_idname = "qup.add_simple_ao"

    def execute(self, context):
        mat = bpy.context.object.active_material
        n = bpy.context.object.active_material.node_tree.nodes.active
        Output_Node = mat.node_tree.nodes.get('Material Output')
        texture_node = n
        bsdf_node = mat.node_tree.nodes.get('Principled BSDF')

        if n.type in tex_nodes:
            file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")
            inner_path = 'NodeTree'
            nodetree_name = 'AO_Simple'

            bpy.ops.wm.append(
                filepath=os.path.join(file_path, inner_path, nodetree_name),
                directory=os.path.join(file_path, inner_path),
                filename=nodetree_name
            )

            ao_node = mat.node_tree.nodes.new(type='ShaderNodeGroup')
            ao_node.node_tree = bpy.data.node_groups['AO_Simple']
            if bsdf_node.location[0] - n.location[0] < 550:
                n.location[0] = bsdf_node.location[0] - 550

            ao_node.location[0] = n.location[0] + 300
            ao_node.location[1] = n.location[1]
            n_link = mat.node_tree.links.new

            n_link(n.outputs[0], ao_node.inputs[0])
            n_link(ao_node.outputs[0], bsdf_node.inputs[0])



            return {'FINISHED'}

class QUP_OT_add_blur(bpy.types.Operator):
    '''Add Simple AO Node '''
    bl_label = "Add blur to Image texture"
    bl_idname = "qup.add_blur"

    def execute(self, context):
        mat = bpy.context.object.active_material
        n = bpy.context.object.active_material.node_tree.nodes.active
        Output_Node = mat.node_tree.nodes.get('Material Output')
        texture_node = n
        bsdf_node = mat.node_tree.nodes.get('Principled BSDF')

        if n.type in tex_nodes:
            file_path = os.path.join(os.path.dirname(__file__), "Gizmo.blend")
            inner_path = 'NodeTree'
            nodetree_name = 'Blur_TMX'

            bpy.ops.wm.append(
                filepath=os.path.join(file_path, inner_path, nodetree_name),
                directory=os.path.join(file_path, inner_path),
                filename=nodetree_name
            )

            blur_node = mat.node_tree.nodes.new(type='ShaderNodeGroup')
            blur_node.node_tree = bpy.data.node_groups['Blur_TMX']


            blur_node.location[0] = n.location[0] - 300
            blur_node.location[1] = n.location[1]
            n_link = mat.node_tree.links.new

            n_link(blur_node.outputs[0], n.inputs[0])




            return {'FINISHED'}
        
# def enum_previews_img_from_directory_items(self, context):
#     pcoll = preview_collections.get("main")
#     if not pcoll:
#         return []
#     if self.my_previews_dir == "":
#         # print("MAKE A NEW THUMB LIST HERE")
#         newlist = []
#         return newlist
#
#     return pcoll.my_previews
def enum_previews_img_from_directory_items(self, context):
    # preview_img_dir_update
    extensions = ('.jpeg', '.jpg', '.png', '.psd', '.tga', '.exr')

    obj = context.object
    if obj.type in {'MESH', 'CURVE'}:
        i_shader = context.object.active_material.node_tree.nodes.active
    if obj.type in {'LIGHT'}:
        i_shader = context.object.data.node_tree.nodes.active

    img = i_shader.image
    head, tail = os.path.split(img.filepath)
    folder = head
    directory = folder

    enum_items = []

    if context is None:
        return enum_items

    pcoll = preview_collections["img_shader"]

    if directory == pcoll.my_previews_dir:
        return pcoll.my_previews

    if directory and os.path.exists(directory):
        # Scan the directory for png files
        image_paths = []
        for fn in os.listdir(directory):
            if fn.lower().endswith(extensions):
                image_paths.append(fn)

        for i, name in enumerate(image_paths):
            # generates a thumbnail preview for a file.
            filepath = os.path.join(directory, name)
            icon = pcoll.get(name)
            if filepath in pcoll:
                enum_items.append((name, name, "", pcoll[filepath].icon_id, i))
            else:
                thumb = pcoll.load(filepath, filepath, 'IMAGE')
                enum_items.append((name, name, "", thumb.icon_id, i))

    pcoll.my_previews = enum_items
    pcoll.my_previews_dir = directory

    return pcoll.my_previews


def preview_texture_enum_update(wm, context):
    global index_prev
    n = context.object.active_material.node_tree.nodes.active
    # n = bpy.context.object

    selected_preview = n.qup.my_previews_texture
    C_name = selected_preview
    i_shader = context.active_node
    img = i_shader.image
    head, tail = os.path.split(img.filepath)
    folder = head

    bpy.ops.image.open(
        filepath=folder + C_name,
        directory=folder,
        files=[{"name": C_name, "name": C_name}], relative_path=True, show_multiview=False)

    my_files = listdir(folder)
    my_files.sort()
    my_files.index(C_name)

    img = bpy.data.images.load(os.path.join(folder, C_name))

    i_shader.image = img
    index_prev.append(my_files.index(C_name))
    print(my_files.index(C_name))
    print(get_default_id())
    # print(index_prev[-1])

    return None


# index_i = [0]
index2 = []

def get_default_id():
    # for n in bpy.context.object.active_material.node_tree.nodes:
    n = bpy.context.object.active_material.node_tree.nodes.active
    if n.type == 'TEX_IMAGE':
        img = n.image
        head, tail = os.path.split(img.filepath)
        folder = head
        my_files = listdir(folder)
        my_files.sort()
        index = my_files.index(tail)
        # if self.my_previews_texture:
        # index2.append(index)
        # return print(index)
        return index

# p=print(get_default_id())

preview_collections = {}
class QupPreview(bpy.types.PropertyGroup,):
    default: StringProperty()


    my_previews_dir: StringProperty(
        name="Folder Path",
        subtype='DIR_PATH',
        default="",
        # update=preview_img_dir_update,
    )

    my_previews_texture: EnumProperty(
        items=enum_previews_img_from_directory_items,
        # default=int(get_default_id()),
        default=0,
        # default=None,
        # default=index2[-1],
        update=preview_texture_enum_update
    )

    pcoll = bpy.utils.previews.new()
    pcoll.my_previews_dir = ""
    pcoll.my_previews = ()

    # preview_collections["main"] = pcoll
    preview_collections["img_shader"] = pcoll


classes = [
    QupPreview,
    QUP_OT_launch_external_soft,
    QUP_OT_reload_img,
    QUP_OT_pack_img,
    QUP_OT_unpack_img,
    QUP_OT_relative_img,
    QUP_OT_add_displace_node,
    QUP_OT_add_bump_node,
    QUP_OT_add_normal_node,
    QUP_OT_add_occlusion_node,
    QUP_OT_replace_onetexturenode,
    QUP_OT_replace_p_bsdf,
    QUP_OT_add_bevel_node,
    QUP_OT_add_simple_ao,
    QUP_OT_add_blur,

    # QUP_MT_pie_images_nodes,

    ]


def register():
    global index_i
    for cls in classes:
        bpy.utils.register_class(cls)
        # bpy.types.WindowManager.qup = bpy.props.PointerProperty(type=QupPreview)
        # bpy.types.Scene.qup = bpy.props.PointerProperty(type=QupPreview)
        # bpy.types.ShaderNode.qup = bpy.props.PointerProperty(type=QupPreview)
        bpy.types.ShaderNodeTexImage.qup = bpy.props.PointerProperty(type=QupPreview)


def unregister():
    global index_i
    for cls in classes:
        bpy.utils.unregister_class(cls)

    del bpy.types.ShaderNodeTexImage.qup

    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)

    preview_collections.clear()
